# OpenProximity2.0 is a proximity marketing OpenSource system.
# Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
from django.conf import settings
from django.utils.translation import ugettext as _
from net.aircable.openproximity.signals import uploader as signals
from net.aircable.utils import getLogger
logger = getLogger(__name__)
from openproximity.models import *

from common import do_upload, is_known_dongle, isAIRcable, found_action

#from pickle import loads

import traceback

def handle(signal, *args, **kwargs):
    logger.info("uploader signal: %s" % signals.TEXT[signal])
    logl = LogLine()
    logl.content += signals.TEXT[signal]
    
    if signal == signals.SDP_RESOLVED:
        logl.content += ' %s:%s' %( kwargs['address'], kwargs['port'])
        handle_sdp_resolved(
                kwargs['dongle'], 
                kwargs['address'], 
                kwargs['port']
        )
    elif signal == signals.SDP_NORECORD:
        logl.content += ' %s' %( kwargs['address'])
        handle_sdp_norecord(
                kwargs['dongle'],
                kwargs['address'])
    elif signal == signals.SDP_TIMEOUT:
        logl.content += ' %s' %( kwargs['address'])
        handle_sdp_timeout(
            kwargs['dongle'],
            kwargs['address'])
    elif signal == signals.FILE_UPLOADED:
        logl.content += ' %s' %( kwargs['address'])
        handle_file_uploaded(
            kwargs['dongle'],
            kwargs['address'],
            kwargs['port'],
            kwargs['files'])
    elif signal == signals.FILE_FAILED:
        logl.content += ' %s, ret:%s' %( kwargs['address'], kwargs['ret'])
        handle_file_failed(
                kwargs['dongle'],
                kwargs['address'],
                kwargs['port'],
                kwargs['files'],
                kwargs['ret'],
                kwargs['stderr'])
    elif signal == signals.PAIR_SUCCESS:
        handle_paired(kwargs['dongle'], kwargs['address'])
    elif signal in [signals.PAIR_TIMEOUT, signals.PAIR_REJECT]:
        handle_pair_error(kwargs['dongle'], kwargs['address'], kwargs['reason'],
            kwargs['exception'])
    else:
        logger.error("signal ignored")
    
    logl.save()

def get_dongles(dongles):
    out = list()
    
    for address in dongles:
        logger.debug(address)
        try:
            if not is_known_dongle(address, UploaderBluetoothDongle) and \
                                                            isAIRcable(address):
                logger.info('not known uploader %s' % address)
                setting = settings.GETUPLOADERDONGLE(address)
                if not setting:
                   logger.info('no settings for uploaders')
                   continue

                logger.info('default settings where found')
                logger.debug(setting)
                max_conn = setting.get('value', 1)
                enabled = setting.get('enable', True)
                name = setting.get('name', _("Autodiscovered Bluetooth dongle"))

                UploaderBluetoothDongle.objects.get_or_create(address=address, 
                    defaults={
                        'name': name,
                        'enabled': enabled,
                        'max_conn': max_conn
                    }
                )
            dongle = UploaderBluetoothDongle.objects.get(address=address, enabled=True)
            out.append( (address, dongle.max_conn, dongle.name) )
        except Exception, err:
            logger.exception(err)
    return out

def handle_sdp_resolved(dongle, remote, channel):
    from openproximity.rpc.server import OpenProximityService
    logger.info("Valid SDP: %s %s" % (remote, channel) )
    remote=RemoteDevice.objects.filter(address=remote).get()
    if RemoteBluetoothDeviceSDP.objects.filter(remote=remote).count() == 0:
        logger.info("New SDP result")
        record = RemoteBluetoothDeviceSDP()
        record.dongle = UploaderBluetoothDongle.objects.get(address=dongle)
        record.channel = channel
        record.remote = remote
        record.save()
    logger.info("Starting bonding process")
    uploader = OpenProximityService.getUploader()
    uploader.start_pairing(remote.address)

def handle_sdp_norecord(dongle, remote):
    logger.info("No SDP: %s" % remote)
    
    from openproximity.rpc.server import OpenProximityService
    OpenProximityService.removePending(remote)

    remote=RemoteDevice.objects.filter(address=remote).get()
    if RemoteBluetoothDeviceNoSDP.objects.filter(remote=remote).count() == 0:
        record = RemoteBluetoothDeviceNoSDP()
        record.dongle = UploaderBluetoothDongle.objects.get(address=dongle)
        record.remote = remote
        record.save()

def handle_sdp_timeout(dongle, remote):
    logger.info("SDP timeout: %s" % remote )

    from openproximity.rpc.server import OpenProximityService
    OpenProximityService.removePending(remote)

    record = RemoteBluetoothDeviceSDPTimeout()
    record.dongle = UploaderBluetoothDongle.objects.get(address=dongle)
    record.setRemoteDevice(remote)
    record.save()

def handle_paired(dongle, remote):
    logger.info("Paired with %s on %s" % (dongle, remote))
    
    record = RemoteBluetoothDevicePairing()
    record.dongle = UploaderBluetoothDongle.objects.get(address=dongle)
    record.setRemoteDevice(remote)
    record.state = RemoteBluetoothDevicePairing.STATES["Passed"]
    record.save()
    found_action(remote, dongle=dongle, record=record)

def handle_pair_error(dongle, remote, msg, exception):
    logger.info("Paired failed %s on %s, %s:%s" % (dongle, remote, msg,
        exception))
    
    record = RemoteBluetoothDevicePairing()
    record.dongle = UploaderBluetoothDongle.objects.get(address=dongle)
    record.setRemoteDevice(remote)
    exc = exception.lower()
    if 'reject' in exc or 'failed' in exc:
        record.state = RemoteBluetoothDevicePairing.STATES["Rejected"]
    else:
        record.state = RemoteBluetoothDevicePairing.STATES["Timeout"]
    record.msg = msg
    record.exception = exception
    record.save()
    
    from openproximity.rpc.server import OpenProximityService
    OpenProximityService.removePending(remote)

def handle_file_uploaded(dongle, remote, channel, files):
    logger.info("files uploaded: %s[%s]: %s" % ( remote, channel, files) )

    from openproximity.rpc.server import OpenProximityService
    OpenProximityService.removePending(remote)

    for camp in get_campaign_rule(files):
        record = RemoteBluetoothDeviceFilesSuccess()
        record.dongle = UploaderBluetoothDongle.objects.get(address=dongle)
        record.campaign = camp
        record.setRemoteDevice(remote)
        record.save()

def handle_file_failed(dongle, remote, channel, files, ret, err):
    logger.info("handle file failed %s[%s]: %s" % (remote, channel, files))
    logger.debug(err)
    try:
        rules = get_campaign_rule(files)
        if rules is None:
            raise Exception("No matching rule for files %s!" % files)
        record = None
        try_again = False

        for rule in get_campaign_rule(files):
            record = RemoteBluetoothDeviceFilesRejected()
            record.dongle = UploaderBluetoothDongle.objects.get(address=dongle)
            record.campaign = rule
            record.ret_value = ret
            record.setRemoteDevice(remote)
            record.save()
            # from here we try again either on timeout or if rejected count is 
            # smaller than filter, for any of the matching rules
            # if only 1 campaign matches we do it all over!
            try_again = try_again | rule.tryAgain(record)

        logger.info("try again: %s" % try_again)
        if try_again:
            from openproximity.rpc.server import OpenProximityService
            uploader = OpenProximityService.getUploader()
            if uploader:
                logger.info("trying again")
                return do_upload(uploader, files, remote)
            else:
                logger.info("no uploader registered")
    except Exception, err:
        logger.error("OOOPS!!!")
        logger.exception(err)
    logger.info("taking out of pending list")

    from openproximity.rpc.server import OpenProximityService
    OpenProximityService.removePending(remote)



"this is the locale selecting middleware that will look at accept headers"

from django.utils.cache import patch_vary_headers
from django.utils import translation
from django.middleware.locale import LocaleMiddleware

from net.aircable.utils import getLogger
logger = getLogger(__name__)

class ILocaleMiddleware(LocaleMiddleware):
    def process_request(self, request):
        try:
            super(ILocaleMiddleware, self).process_request(request)
        except  Exception, err:
            logger.exception(err)



#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
from django.conf.urls.defaults import *
from django.contrib.auth.views import login, logout
from django.contrib.staticfiles.urls import staticfiles_urlpatterns

from net.aircable.utils import getLogger
logger = getLogger(__name__)

# Uncomment the next two lines to enable the admin:
from django.contrib import admin

import os

from net.aircable.openproximity.pluginsystem import pluginsystem

admin.autodiscover()

urlpatterns = patterns ('',
    (r'^accounts/$', login, {'template_name': 'op/login.html'}),
    (r'^accounts/login', login, {'template_name': 'op/login.html'}),
    (r'^accounts/', include('django.contrib.auth.urls')),

    # include needed interfaces    
#    (r'^bluez/', include('bluez.urls')),
    (r'^openproximity/', include('openproximity.urls')),
    
    #include translation site
    (r'^admin/translate/', include('rosetta.urls'),{},"translate"),
    
    # Uncomment the admin/doc line below and add 'django.contrib.admindocs' 
    # to INSTALLED_APPS to enable admin documentation:
#    (r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    (r'^admin/', include(admin.site.urls),{},"admin"),
    
    (r'^site_media/(?P<path>.*)$', 'django.views.static.serve',
        {       'document_root': os.path.join(os.path.dirname(__file__), 'media')},
        "site-media"
    ),
    
    (r'^notification/', include('notification.urls'),{},"notification"),
)

for plugin in pluginsystem.get_plugins('urls'):
    logger.info("%s provides urls" % plugin.provides.get('name', plugin.name))
    url, path = plugin.provides.get('urls')
    urlpatterns += patterns( '', 
        (r'^%s/' % url, 
            include('%s.%s' % ( plugin.name, path)),
            {},
            plugin.name # allow url to be reverse resolved
    ))

urlpatterns += patterns('', 
    (r'', include('openproximity.urls'),{},"openproximity")
)

urlpatterns += staticfiles_urlpatterns()



# -*- coding: utf-8 -*-
#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from lxml import etree
from re import compile
from exceptions import IOError

from net.aircable.utils import getLogger
logger = getLogger(__name__)

VALID_ADDRESS=compile("([0-9a-f]{2}\:){5}([0-9a-f]{2})")

DEFAULT='''<xml>
<dongle>
</dongle>
</xml>'''

class XMLTool:
    """
    In [1]: from lxmltool import XMLTool

    In [2]: xt = XMLTool()

    In [3]: tree = xt.getXmlTree()

    In [4]: type(tree)
    Out[4]: <type 'lxml.etree._ElementTree'>

    In [5]: xt.getAdminPasswd()
    Out[5]: 'secret\n    '

    In [6]: xt.getAd
    xt.getAdminEmail   xt.getAdminName    xt.getAdminPasswd  

    In [6]: xt.getAdminName()
    Out[6]: 'x-ip\n    '

    In [7]: xt.getAdminEmail()
    Out[7]: 'foo@bar.com\n    '
    """
    
    def __init__(self, xmlfile="/etc/openproximity/settings.xml"):
        self.__file = xmlfile
        self.tree = None

    def __getXmlTree(self):
        """ Open an xml file and return an etree xml instance or None
        """
        self.tree = etree.parse(self.__file)
        if self.tree is None:
            raise Exception("no config file")
        
    def __sanitize(self):
        try:
            if self.tree is None:
                self.__getXmlTree()
        except Exception, err:
            logger.info("failed while loading file settings, trying to simulate"
                    " config file")
            logger.exception(err)
            self.tree=etree.fromstring(DEFAULT)

    def __getValueOrDefault(self, key, default):
        self.__sanitize()
        try:
            return self.tree.xpath(key)[0].text
        except:
            # if we got here then it's quite possible there's no 
            # setting defined in the xml
            return default

    #persistance methods
    def getAllSettings(self):
        return {
                'admin':{
                    'name': self.getAdminName(),
                    'password': self.getAdminPasswd(),
                    'email': self.getAdminEmail()
                },
                'logo': self.getLogo(),
                'debug': self.isDebugEnabled(),
                'translate': self.isTranslateEnabled(),
                'dongles': self.getAllDonglesSettings()
        }

    def genXML(self, settings, indent=1, header="xml"):
        out = ""
        if header:
            out+="<%s>\n" % header
        for key, value in settings.iteritems():
            for i in range(indent):
                out+="\t"
            out+="<%s>" % key
            if type(value) is dict:
                out+="\n%s\n" % self.genXML(value, indent+1, None)
                for i in range(indent):
                    out+="\t"
                out+="</%s>\n" % key

            else:
                out+=str(value)
                out+="</%s>\n" % key
        if header:
            out+="</%s>" % header

        return out
        
    def saveSettings(self, settings):
        out = file(self.__file, 'w')
        out.write(self.genXML(settings))
        out.close()
        
    # generic functions
    def getValueOrDefault(self, key, default=None):
        """Generic return function"""
        return self.__getValueOrDefault(key, default)
        
    def getDict(self, parent, default={}):
        """Generic access to dict values"""
        self.__sanitize()
        key = self.tree.findall(parent)
        if len(key) > 0:
            return self.__todict(key[0], False)
        return default

    # Admin stuff...
    def getAdminPasswd(self, default="aircable"):
        """ Return the admin password set in the xml file as string or None
        """
        return self.__getValueOrDefault('admin/password', default)

    def getAdminName(self, default="admin"):
        """ Return the admin name set in the xml file as string or None
        """
        return self.__getValueOrDefault('admin/name', default)

    def getAdminEmail(self, default="foo@foo.com"):
        """ Return the admin email set in the xml file as string or None
        """
        return self.__getValueOrDefault('admin/email', default)
            
    # logo staff ...
    def getLogo(self, default='logo.gif'):
        """ Return the path to the logo file as string
        """
        return self.__getValueOrDefault('logo', default)

    # debug staff
    def isDebugEnabled(self, default="true"):
        """Return debug preference as configured in xxx.xml
        """
        debug = self.__getValueOrDefault('debug', default)
        return debug.lower() == "true"

    def isTranslateEnabled(self, default="true"):
        """Return translate preference as configured in xxx.xml
        """
        translate = self.__getValueOrDefault('translate', default)
        return translate.lower() == "true"

    # dongle stuff ...
    def getAllDonglesSettings(self):
        try:
            self.__sanitize()
            blocks = self.tree.findall('dongle/block')
            out = dict()
            for block in blocks:
                addr = block.find('address').text
                out[addr]=self.__todict(block)

            # back to default
            default = self.tree.findall('dongle/default')
            if len(default) > 0:
                out['default'] = self.__todict(default[0])
            return out
        except AttributeError:
            return {}
    
    def getSettingsByAddress(self, address=""):
        """
        In [1]: from lxmltool import XMLTool

        In [2]: xt = XMLTool()

        In [3]: xt.getSettingsByAddress()
        Out[3]: {}

        In [4]: xt.getSettingsByAddress('f4:50:C2:00:00:02')
        Out[4]: {'scanner': ['1', 'True'], 'uploader': ['1', 'False']}

        In [5]: dic = xt.getSettingsByAddress('f4:50:C2:00:00:02')

        In [6]: dic
        Out[6]: {'scanner': ['1', 'True'], 'uploader': ['1', 'False']}

        In [7]: dic['scanner']
        Out[7]: ['1', 'True']

        In [8]: dic['uploader']
        Out[8]: ['1', 'False']
        
        # if the address doesnt exist, return default values declared in the
        # xml file
        In [9]: xt = XMLTool()

        In [9]: xt.getSettingsByAddress('f4:50:C2:00:00:06')
        Out[9]: {'scanner': ['1337', 'False'], 'uploader': ['20', 'True']}

        """
        
        if not VALID_ADDRESS.match(address.lower()):
            raise Exception("Not Valid Bluetooth Address %s" % address)
        try:
            self.__sanitize()

            blocks = self.tree.findall('dongle/block')
            for block in blocks:
                addr = block.find('address').text.lower()
                if address.lower().startswith(addr):
                    logger.info("%s passes filter %s" %( address, addr))
                    return self.__todict(block)

            # back to default
            default = self.tree.findall('dongle/default')
            if len(default) > 0:
                block = default[-1]
                return self.__todict(block)
            
            # not even default settings
            return {}
            
        except AttributeError:
            return {}

    def __todict(self, block, ignore_address=True):
        out = dict()
        for children in list(block):
            if children.tag!='address' or not ignore_address:
                if len(list(children)) > 0:
                    out[children.tag] = self.__todict(children, ignore_address)
                else:
                    out[children.tag]=children.text
        return out

if __name__ == '__main__':
    print "using foo.xml"
    xt = XMLTool("foo2.xml")

    print '00:50:C2:00:00:02', xt.getSettingsByAddress('00:50:C2:00:00:02')
    print 'f4:AA:BB:00:00:06', xt.getSettingsByAddress('f4:AA:BB:00:00:06')
    print xt.getAdminName()
    print xt.getAdminPasswd()
    print xt.getAdminEmail()
    set = xt.getAllSettings()
    print set
    print xt.genXML(set)




#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from django.core import serializers
from django.http import HttpResponse
from django.shortcuts import render_to_response

class AlreadyRegistered(Exception):
    pass

class TweetSite(object):
    """
        A tweet site
    """
    
    def __init__(self, klass, field="pk", convert=int):
        '''
            Creates a new TweetSite.
            arguments:
                klass: which class to use
                field: which field to use for sorting
                convert: converting function when using non integers variables
        '''
        self.klass = klass
        self.field = field
        self.convert = convert
        
        from django.conf.urls.defaults import patterns, url, include
        
        self.urlpatterns = patterns( '',
            url(r'^latest/', self.latest_view),
            url(r'^posts/', self.posts_view),
            url(r'.*', self.index_view), 
        )

    def latest_view(self, request):
        """
            This view will return which is the latest entry in the table.
        """
        try:
            data = serializers.serialize('json',(self.klass.objects.latest('pk'),))
        except:
            data = None
        return HttpResponse(data, mimetype="application/json")

    def posts_view(self, request):
        """
            This view will return a json array with the latest posts
        """
        
        pk = request.GET.get('pk', None)
        amount = int(request.GET.get('amount', 10))
        
        qs = self.klass.objects.all()
        
        if pk is not None:
            pk = self.convert(pk)
            
            filt = { '%s__gt' % self.field: pk }
        
            qs = qs.filter(**filt)
        qs = qs.order_by('-%s' % self.field)
        
        data = serializers.serialize('json', qs[:amount])
        
        return HttpResponse(data, mimetype="application/json")
        
    def index_view(self, request):
        return render_to_response( 'microblog/base.html',{})



# -*- coding: utf-8 -*-
import django, os, sys
from configglue.schema import ( Section, StringOption, IntOption, BoolOption,
                                DictOption, TupleOption, ListOption )
from django_configglue.schema import Django13Schema
from ConfigParser import MAX_INTERPOLATION_DEPTH

class _Context(object):
    def __init__(self, config, section):
        self.config = config
        self.section = section

    def __getitem__(self, key):
        if '.' in key:
            return self.config.get(*key.split('.'))
        else:
            return self.config.get(self.section, key)

def _interpolate(self, section, option, rawval, vars):
      # do the string interpolation
      value = rawval
      depth = MAX_INTERPOLATION_DEPTH
      while depth:                    # Loop through this until it's done
          depth -= 1
          if "%(" in value:
              value = self._KEYCRE.sub(self._interpolation_replace, value)
              try:
                  value = value % _Context(self, section)
              except KeyError, e:
                  raise InterpolationMissingOptionError(
                      option, section, rawval, e.args[0])
          else:
              break
      if "%(" in value:
          raise InterpolationDepthError(option, section, rawval)
      return value

class OpenProximitySchema(Django13Schema):
    ################################################
    # openproximity general settings
    ################################################
    openproximity = Section('openproximity')
    openproximity.store_path = StringOption( default='/var/lib/openproximity',
        help = 'Path were we store the database file (if using sqlite) and storing files for campaigns etc')
    openproximity.op_debug = BoolOption(default=True,
        help = 'Disable if you want to hide the "Databrowse" tab')
    openproximity.op_translate = BoolOption(default=True,
        help = 'Disable if you want to hide the "Translate" tab')
    openproximity.op_twitter = BoolOption(default=True,
        help = 'Disable if you want to hide the Twitter news client')
    openproximity.op_logo = StringOption(default='logo.gif',
        help = 'Logo to display instead of AIRcable logo')
    openproximity.op_plugins = DictOption(item=BoolOption(),
        help="""A list of plugins name with they're enable/disable state overrides plugins defaults)""")
    openproximity.op_scanners = ListOption( item=TupleOption(4),
        help="""A list of ([MAC filter], scanner priority) for dongle configuration on initial discovery""")
    openproximity.op_uploaders = ListOption( item=TupleOption(4),
        help="""A list of ([MAC filter], max connections) for dongle configuration on initial discovery""")

    ################################################
    # cherrypy settings
    ################################################
    cherrypy = Section('cherrypy')
    cherrypy.cp_host = StringOption(default="localhost",
                                                  help='hostname to listen on')
    cherrypy.cp_port = IntOption(default=8088, help='port to listen on')
    cherrypy.cp_server_name = StringOption(default="localhost",
                                    help="CherryPy's SERVER_NAME environ entry")
    cherrypy.cp_daemonize = BoolOption(default = False,
                                        help='whether to detach from terminal')
    cherrypy.cp_pidfile = StringOption(default=None,
                              help="write the spawned process-id to this file")
    cherrypy.cp_workdir = StringOption(default=None,
                              help="change to this directory when daemonizing")
    cherrypy.cp_threads = IntOption(default=20,
                                    help='Number of threads for server to use')
    cherrypy.cp_ssl_certificate = StringOption(default=None,
                                                help="SSL certificate filename")
    cherrypy.cp_ssl_private_key = StringOption(default=None,
                                                help="SSL private key filename")
    cherrypy.cp_server_user = StringOption(default='www-data',
                                          help="user to run daemonized process")
    cherrypy.cp_server_group = StringOption(default='www-data',
                                            help="group to daemonized process")

    ################################################
    # rpyc settings
    ################################################
    rpyc = Section('rpyc')
    rpyc.rpc_host = StringOption(default="localhost",
                                                  help='hostname to listen on')
    rpyc.rpc_port = IntOption(default=8010, help='port to listen on')
    rpyc.rpc_daemonize = BoolOption(default = False,
                                        help='whether to detach from terminal')
    rpyc.rpc_threaded = BoolOption(default = True,
                                    help='run in threaded way instead of forked')
    rpyc.rpc_autoregister = BoolOption(default = False,
                                help='try registering the server in name servers')
    rpyc.rpc_pidfile = StringOption(default = None,
                              help="write the spawned process-id to this file")
    rpyc.rpc_server_user = StringOption(default='www-data',
                                          help="user to run daemonized process")
    rpyc.rpc_server_group = StringOption(default='www-data',
                                            help="group to daemonized process")

    ################################################
    # debugging settings
    ################################################
    debug = Section('debug')
    debug.debug_console = StringOption(default = 'INFO',
        help='console debug level, default INFO')
    debug.debug_level = StringOption(default = 'DEBUG',
        help='in file debug level, default DEBUG')
    debug.debug_path = StringOption(default='/var/log/openproximity',
        help='default path used for storing log files')
    debug.debug_maxsize = IntOption(default=10, 
        help='max log file in MB, defaults to 10')
    debug.debug_count = IntOption(default=5,
        help='amount of log files to store when rotating, defaults to 2')
    debug.debug_filename=StringOption(default=None,
        help='file name used for this debug session, defaults to sys.argv[1]')
    debug.debug_console_format = StringOption(
        default='%(name)s %(module)s:%(funcName)s: %(message)s',
        help='console log formatting')
    debug.debug_format = StringOption(
        default='%(asctime)-12s - %(levelname)5s - %(name)10s - %(funcName)-12s: %(message)s',
        help='file log formatting')
    debug.debug_disables = StringOption( default='', 
        help='comma separated list of disable log modules')

    rpc_client = Section("rpc_client")
    rpc_client.client_mode = StringOption()
    rpc_client.client_daemonize = BoolOption(default=False)
    rpc_client.client_pidfile = StringOption(default=None)
    rpc_client.client_host = StringOption(default="localhost")
    rpc_client.client_port = IntOption(default=8010)

if os.path.isfile('/etc/timezone'):
    OpenProximitySchema.django.time_zone.default = \
        file('/etc/timezone').readline().strip()

OpenProximitySchema.django.auth_profile_module = StringOption(default=None,
                              help="Module to use for authoring")
OpenProximitySchema.django.serialization_modules = DictOption(
                              help="Default empty dict")



#!/usr/bin/env python
#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import setpaths

try:
    import settings # Assumed to be in the same directory.
except ImportError, e:
    import sys, traceback
    print "Error: Can't find the file 'settings.py' in the directory",
    print "containing %r. It appears you've customized things.\nYou'll have" % __file__,
    print "to run django-admin.py, passing it your settings module.\n(If the",
    print "file settings.py does indeed exist, it's causing an ImportError",
    print "somehow.)""" 
    print traceback.print_exc()
    sys.exit(1)

from django.core.management import execute_from_command_line, setup_environ
setup_environ(settings)

from net.aircable.openproximity.pluginsystem import pluginsystem
pluginsystem.post_environ()

def main():
    from django.conf import settings as set
    execute_from_command_line()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# -*- coding: utf-8 -*-
import dbus, dbus.glib, gobject, select
import os, sys, errno
import rpyc

from rpyc import async
from net.aircable import autoreload
from net.aircable.openproximity.pluginsystem import pluginsystem
from net.aircable.utils import getLogger
logger = getLogger(__name__)

server = None
manager = None
bus = None
loop = None

def poll(fd, condition):
    '''
    This function gets called whenever there's data waiting in the incomming 
    socket, so we can flush the data from the server.
    '''
    try:
        server.poll()
        return True
    except EOFError, eof:
        logger.error("EOF while polling %s" % eof)
        logger.exception(eof)
        stop()
        return False 
    except Exception, err:
        logger.error("error during poll %s" % err)
        logger.exception(err)
    return True

def stop():
    '''
    Safe stop function
    '''
    global manager
    from uploader import UploadManager
    if getattr(manager, 'exposed_stop', None):
        manager.exposed_stop()
    loop.quit()

def handle_name_owner_changed(own, old, new):
    '''
    Will get called whenever a name owner changes in dbus.
    If the change is with the BlueZ name (either it started or ended) then
    we stop the rpc client so it restarts.
    '''
    if own.startswith('org.bluez'):
        if new is None or len(str(new))==0:
            logger.info( "bluez has gone down, time to get out")
        else:
            logger.info( "bluez started, time to restart")
        stop()

def handle_adapter_added_or_removed(path, signal):
    '''
    When ever a dongle is added or removed this function gets called, so we can
    restart the rpc client.
    '''
    logger.info("bluez.%s: %s" % (signal, path))
    stop()

def init(mode):
    '''
    Gets called from inside the gobject loop once dbus is accessible so we can
    complete the init process.
    '''
    global manager, bus, loop, server
    
    logger.info("init")
    
    # get the list of connected dongles
    a=list()
    for b in manager.manager.ListAdapters():
        obj = bus.get_object('org.bluez', b)
        adapter = dbus.Interface(obj, 'org.bluez.Adapter')
        a.append(str(adapter.GetProperties()['Address']))
    logger.info("Connected dongles: %s" % a)

    # do the remote registration.
    register={
        'scanner': async(server.root.scanner_register),
        'uploader': async(server.root.uploader_register),
    }.get(mode, async(server.root.generic_register))

    register(
        remote_quit=stop, 
        client=manager, 
        dongles=a, 
    )
    logger.info("exiting init")

    # stupid way to make rpyc do something
    str(server.root)

    return False

def run(host, port, mode):
    '''
        Will get called from a child process once the autoreload system is ready
        everything will be setup so the rpc client can work.
    '''
    global server, manager, bus, loop
    
    loop=gobject.MainLoop()
    
    # first connect to the server, we can't do much without it
    try:
        server = rpyc.connect(host, int(port))
    except:
        import time
        logger.info("server is not running")
        time.sleep(10)
        autoreload.RELOAD = True
        sys.exit(3)

    # now try to get into dbus and init gobject
    bus=dbus.SystemBus()
    gobject.threads_init()
    dbus.glib.init_threads()

    # register for a few signals
    bus.add_signal_receiver(
        handle_name_owner_changed,
        'NameOwnerChanged',
        'org.freedesktop.DBus',
        'org.freedesktop.DBus',
        '/org/freedesktop/DBus'
    )

    bus.add_signal_receiver(
        handle_adapter_added_or_removed,
        signal_name='AdapterAdded',
        dbus_interface='org.bluez.Manager',
        member_keyword='signal' 
    )

    bus.add_signal_receiver(
        handle_adapter_added_or_removed,
        signal_name='AdapterRemoved',
        dbus_interface='org.bluez.Manager',
        member_keyword='signal'
    )

    # create the manager and register for init
    try:
        if mode == 'scanner':
            from scanner import ScanManager
            logger.info("init scanner")
            manager = ScanManager(bus, rpc=server)
        elif mode == 'uploader':
            logger.info("init uploader")
            from uploader import UploadManager
            manager = UploadManager(bus, rpc=server, loop=loop)
        else:
            for i in pluginsystem.get_plugins('serverxr'):
                if mode == i.provides['serverxr_type']:
                    logger.info("init %s" % i.provides['serverxr_type'])
                    module = __import__("%s.serverxr" % i.name, 
                                        fromlist=[
                            i.provides['serverxr_manager'],])
                    klass = getattr(module, i.provides['serverxr_manager'])
                    manager = klass (bus, rpc=server)
                    break
        if manager is None:
            raise Exception ("Not valid type")
        gobject.timeout_add(100, init, mode) 
        # delay initialization 'til loop is running
    except dbus.DBusException, err:
        logger.info("bluez isn't ready, delaying init")

    # start our loop and setup ping
    flags = gobject.IO_IN | gobject.IO_ERR | gobject.IO_HUP
    gobject.io_add_watch(server.fileno(), flags, poll)
    # give control to the mainloop
    loop.run()

    # if we get here by any how then it's time to restart
    autoreload.RELOAD = True

if __name__ == '__main__':
    from django.core.management import execute_manager, setup_environ
    import settings
    setup_environ(settings)

    # get a list of valid modes
    valid_modes=['scanner', 'uploader']

    valid_modes.extend(
        [ i.provides['serverxr_type'] 
            for i in pluginsystem.get_plugins('serverxr') ] )

    if sys.argv[3] not in valid_modes:
        print "not valid type, you can use any of", valid_modes
        sys.exit(0)

    run(*sys.argv[1:4])



#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from django.core.management.base import BaseCommand
from django.conf import settings
from django.utils.importlib import import_module

from net.aircable.utils import getLogger
logger = getLogger(__name__)

class Command(BaseCommand):
    help = "try running a command inside an egg, pass as argument the wanted command"

    def handle(self, command="", *args, **kwargs):
        logger.debug("searching for %s command" % command)
        for app in settings.INSTALLED_APPS:
            logger.debug("trying with %s" % app)
            try:
                mod = import_module("%s.management.commands.%s" % (app, command))
                logger.debug("found it!")
                return mod.Command().handle(*args, **kwargs)
            except :
                continue

        logger.info("Not provided")
        raise Exception, "%s is not available" % command



#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
from django import forms
from re import compile
from models import MarketingCampaign

AIRCABLE_MAC=['00:50:C2', '00:25:BF']
ADDRESS_MAC=compile("([0-9A-F]{2}\:){5}([0-9A-F]{2})")

def isAIRcable(address):
    return address[:8].upper() in AIRCABLE_MAC

class DongleForm(forms.Form):
    address = forms.CharField()
    name = forms.CharField(max_length=20)
    scan = forms.BooleanField(required=False)
    scan_pri = forms.IntegerField(required=False)
    upload = forms.BooleanField(required=False)
    upload_max = forms.IntegerField(required=False)

    def clean_address(self):
        address = self.cleaned_data['address']
        if ADDRESS_MAC.match(address):
            self.isAIRcable=isAIRcable(address)
            return address
        raise forms.ValidationError("Not a Bluetooth Address")

    def clean_upload(self):
        isAIR = getattr(self, 'isAIRcable', None)
        upload = self.cleaned_data['upload']
        if not isAIR and upload:
            # if you're reading this that means you can read some code
            # and you might want to figure out how to use OP2 with non aircable
            # dongles. Well I'm not going to tell you to remove this lines
            # I'm only going to tell you that this are not the only. You should
            # check code under op2/serverXR too :D
            # MN
            raise forms.ValidationError("Only AIRcable dongles are allowed to upload")
        return self.cleaned_data['upload'] 

class CampaignForm(forms.ModelForm):
    friendly_name = forms.CharField(max_length=100) # dont' want a multi line field here
    
    class Meta:
        model = MarketingCampaign
        #fields = ('friendly_name',)




#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


from django import template
from django.conf import settings
from net.aircable.openproximity.pluginsystem import pluginsystem

register=template.Library()

def do_settings(parser, token):
    return SettingsNode()

class SettingsNode(template.Node):
    def render(self, context):
        context['settings'] = {
            'logo': settings.OP_LOGO,
            'debug': settings.OP_DEBUG,
            'translate': settings.OP_TRANSLATE,
            'twitter': settings.OP_TWITTER,
        }
        return ''

register.tag('settings', do_settings)

def do_plugins(parser, token):
    return PluginsNode()

class PluginsNode(template.Node):
    def render(self, context):
        context['plugins'] = pluginsystem.get_plugins('urls')
        return ''

register.tag('plugins', do_plugins)

@register.simple_tag
def createNavButton(ref, rid, text, style=None):
    return '''<div class="nav_button button" 
            id="%(id)s" onclick="%(href)s">%(text)s</div>''' % { 
                'id': rid, 'href': ref, 'text': text, 'style': style or ''}



# -*- coding: utf-8 ; Mode: python; tab-width: 4 ; indent-tabs-mode: nil -*-
# OpenProximity2.0 is a proximity marketing OpenSource system.
# Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


'''
OpenProximity pairing agent, this agent is ment to be run along our uploader
process.
'''
import gobject
import sys, os
import dbus, dbus.service
import rpyc

from net.aircable.utils import getLogger
logger = getLogger(__name__)

PATH="/net/aircable/agent"
# Path where we register our pairing agent.

bus = dbus.SystemBus()

class Rejected(dbus.DBusException):
    '''
    When the passkeys don't match BlueZ expects us to raise an exception.
    '''
    _dbus_error_name = "org.bluez.Error.Rejected"

class Agent(dbus.service.Object):
    '''
    The passkey agent it self.
    
    Documentation for the methods come from BlueZ docs.
    '''

    AGENTS = dict()

    @classmethod
    def getAgent(klass, dongle):
        logger.info("Agent request for dongle %s" % dongle)
        if dongle not in klass.AGENTS:
            raise Exception("You need to initializate the dongles first")

        for agent in klass.AGENTS[dongle]:
            if not agent.busy:
                agent.busy = True
                logger.info("Going to use %s" % agent.path)
                return agent
        raise Exception("No agents are available")

    def __init__(self, rpc_server, dongle, *a, **kw):
        self.rpc_server = rpc_server
        dbus.service.Object.__init__(self, *a, **kw)
        if not dongle in Agent.AGENTS:
            Agent.AGENTS[dongle]=list()
        self.index=len(Agent.AGENTS[dongle])
        Agent.AGENTS[dongle].append(self)
        self.path="%s/%s/%i" % (PATH, dongle, self.index)
        logger.info("Started agent for path %s" % self.path)
        self.busy = False

    @dbus.service.method("org.bluez.Agent", in_signature="", out_signature="")
    def Release(self):
        '''
        Release the agent, and exit the loop. Gets called by BlueZ when it 
        needs us to release the agent.
        '''
        logger.info("Agent Release %s" % self.path)
        self.busy = False

    @dbus.service.method("org.bluez.Agent", in_signature="os", out_signature="")
    def Authorize(self, device, uuid):
        '''
        This method gets called when the service daemon needs to authorize a 
        connection/service request.
        Don't raise any exception so BlueZ knows we accept.
        '''
        logger.info("Authorize (%s, %s)" % (device, uuid))

    @dbus.service.method("org.bluez.Agent", in_signature="o", out_signature="s")
    def RequestPinCode(self, path):
        '''
        This method gets called when the service daemon needs to get the 
        passkey for an authentication.

        The return value should be a string of 1-16 characters length. The 
        string can be alphanumeric.
        '''
        logger.info("request pin code")
        try:
            device = dbus.Interface(bus.get_object("org.bluez", path), 
                            "org.bluez.Device")
            dongle = dbus.Interface(bus.get_object(
                "org.bluez", device.GetProperties()['Adapter']),
                                "org.bluez.Adapter")
            device=str(device.GetProperties()['Address'])
            dongle=str(dongle.GetProperties()['Address'])
            pin=self.rpc_server.root.getPIN(device, dongle)
            logger.info("RequestPinCode (%s->%s): %s" % (dongle, device, pin) )
            return pin
        except Exception, err:
            logger.error(err)
            return "1234"

    @dbus.service.method("org.bluez.Agent", in_signature="o", out_signature="u")
    def RequestPasskey(self, device):
        '''
        This method gets called when the service daemon needs to get the 
        passkey for an authentication.

        The return value should be a numeric value between 0-999999.
        '''
        logger.info("RequestPasskey")
        logger.info("RequestPasskey (%s): %s" % (device, PIN) )
        return dbus.UInt32(PIN)

    @dbus.service.method("org.bluez.Agent", in_signature="ou", out_signature="")
    def DisplayPasskey(self, device, value):
        '''
        This method gets called when the service daemon needs to display a 
        passkey for an authentication.

        The entered parameter indicates the number of already typed keys on the 
        remote side.

        An empty reply should be returned. When the passkes needs no longer to 
        be displayed, the Cancel method of the agent will be called.

        During the pairing process this method might be called multiple times 
        to update the entered value.
        '''
        logger.info("DisplayPasskey (%s, %s)" % (device, value))

    @dbus.service.method("org.bluez.Agent", in_signature="ou", out_signature="")
    def RequestConfirmation(self, device, passkey):
        '''
        This method gets called when the service daemon needs to confirm a 
        passkey for an authentication.

        To confirm the value it should return an empty reply or an error in 
        case the passkey is invalid.
        '''
        logger.info("RequestConfirmation (%s, %s)" % (device, passkey))
        device = dbus.Interface(bus.get_object("org.bluez", device),
                        "org.bluez.Device")
        dongle = dbus.Interface(bus.get_object("org.bluez", 
                            device.GetProperties()['Adapter']),
                "org.bluez.Adapter")
        device=str(device.GetProperties()['Address'])
        dongle=str(dongle.GetProperties()['Address'])
        pin=self.rpc_server.root.getPIN(device, dongle)
        if passkey == pin:
            logger.info("passkey matches")
            return
        logger.info("passkey doesn't match")
        raise Rejected("Passkey doesn't match")

    @dbus.service.method("org.bluez.Agent", in_signature="s", out_signature="")
    def ConfirmModeChange(self, mode):
        '''
        This method gets called if a mode change is requested that needs to be 
        confirmed by the user. An example would be leaving flight mode.
        '''
        logger.info("ConfirmModeChange (%s)" % (mode))

    @dbus.service.method("org.bluez.Agent", in_signature="", out_signature="")
    def Cancel(self):
        '''
        This method gets called to indicate that the agent  request failed 
        before a reply was returned.
        '''
        logger.info("Cancel")



from django.template.loaders.app_directories import Loader

class ReversedLoader(Loader):
    def get_template_sources(self, template_name, template_dirs=None):
        vals = list(
            Loader.get_template_sources(self, template_name, template_dirs)
        )
        vals.reverse()
        return vals



try:
    from net.aircable.utils import getLogger
    logger = getLogger(__name__)
except:
    import logging
    logging.basicConfig(level=logging.DEBUG)
    logger = logging.getLogger("")

from threading import Timer, Thread, Event, Condition, Lock
import time
from time import time as now

class Task:
    def internal_callback(self):
        logger.debug("Timer timeout %s" % self)
        try:
            self.callback(*self.a, **self.kw)
        except Exception, err:
            logger.error("Exception during callback")
            logger.exception(err)
        Dispatcher.cueue.remove(self)
    
    def cancel(self):
        logger.info("Timer.cancel %s" % self)
        Dispatcher.cueue.remove(self)

    def __init__(self, triggerTime, callback, *a, **kw):
        assert type(triggerTime) is float or int
        assert callable(callback)
        logger.info("new Timer %s %f %s" % (self, triggerTime, callback))
        self.callback = callback
        self.a = a
        self.kw = kw
        self.triggerTime = triggerTime

class Dispatcher(Thread):
    cueue = None
    instance = None
    cond = None
    nextTrigger = None

    def __init__(self):
        Thread.__init__(self)
        Dispatcher.instance = self
        Dispatcher.cueue = []
        self.cond = Condition(Lock())
        self.evt = Event()
        self.daemon = True
    
    def run(self):
        while True:
            logger.debug("sleeping for %s" % self.nextTrigger)
            self.evt.wait(self.nextTrigger)
            n = now()
            tasks = [ b for b in Dispatcher.cueue if b.triggerTime <= n ]
            logger.debug("Doing %i callbacks" % len(tasks))
            for t in tasks:
                t.internal_callback()
            if len(Dispatcher.cueue) > 0:
                self.nextTrigger = min([b.triggerTime for b in Dispatcher.cueue]) - now()
            else:
                self.nextTrigger = None
            self.evt.clear()


    def __callLater(self, triggerTime, callback, *a, **kw):
        t = Task(triggerTime, callback, *a, **kw)
        Dispatcher.cueue.append(t)
        if not self.nextTrigger or triggerTime < self.nextTrigger:
            logger.debug("notifying condition")
            # we need to trigger before the next event
            self.evt.set()
            self.nextTrigger = triggerTime
        return t

    @classmethod
    def getPendings(klass):
        return klass.cueue
    
    @classmethod
    def callLater(klass__, triggerTime, callback, *a, **kw):
        return klass__.instance.__callLater(triggerTime, callback, *a, **kw)

    @classmethod
    def cancelCallLater(klass, task):
        if task in klass.cueue:
            task.cancel()
            return True
        return False

    @classmethod
    def cancelAll(klass):
        for task in klass.cueue:
            task.cancel()
        klass.cueue = []
Dispatcher().start()

if __name__ == '__main__':
    import sys
    t1=t2=t3=None
    def task1():
        print now(), "task1 completed"
        sys.exit(0)
    
    def task2(*a, **kw):
        print now(), "task2 completed", a, kw
    
    def task3():
        print now(), "task3 completed"
    
    def task4(t):
        Dispatcher.callLater(now(), task2, "hello world", a=1, b=2, c=3)
        print now(), "task4 completed", t, Dispatcher.cancelCallLater(t)
    
    t1=Dispatcher.callLater(10+now(), task1)
    t2=Dispatcher.callLater(1+now(), task2, "hello world", a=1, b=2, c=3)
    t3=Dispatcher.callLater(11+now(), task3)
    Dispatcher.callLater(4+now(), task4, t3)
    Dispatcher.callLater(2+now(), task4, None)
    Dispatcher.callLater(3+now(), task4, t2)
    
    print now(), "started"



from setuptools import setup, find_packages
try:
    from openproximity import __version__
except:
    import sys
    sys.path.append('src')
    from openproximity import __version__
    sys.path.remove('src')


PACKAGES=find_packages('src')+['locale']
REQUIRES=[
    'simplejson',
    'lxml',
    #'dbus-python',
    'django>=1.3',
    'openproximity-external-media',
    'wadofstuff-django-serializers-op',
    'PyOFC2-op',
    'rpyc-op',
    'aircable-library-op',
    'django-notification-op',
    'django-cpserver-op',
    'openproximity-plugin-test',
    'rpyc-op',
    'django-configglue',
    'south',
    'django-rosetta',
    'django-mailer',
    'django-extensions',
    'django-restapi-op',
    'poster',
    'django-timezones-op',
]

setup(name = "OpenProximity",
      version = __version__,
      author = "Naranjo Manuel Francisco",
      author_email = "manuel@aircable.net",
      description = ("OpenProximity an OpenSource proximity marketing solution"),
      license = "Apache V2",
      packages = PACKAGES,
      package_dir = { '': 'src'},
      keywords = "openproximity bluetooth obex proximity marketing",
      url = "https://github.com/OpenProximity/web",
      include_package_data = True,
      install_requires = REQUIRES,
      zip_safe = True,
      entry_points = {
        'console_scripts': [
            'OpenProximity-manage = openproximity.lib.manage:main',
            ],
        },
)

#    package_data={
#        "openproximity": [
#            "src/*.cfg",
#            "src/locale/*",
#            "src/microblog/templates/*",
#            "src/openproximity/templtates/*",
#            "src/openproximity/static/*"
#        ]
#    },



#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
# Create your views here.

# imports from stdlib
from re import compile
from datetime import datetime
from mimetypes import guess_type as guess_mime
import rpyc, os, time

# imports from django
from django.db import models as mod
from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.template.loader import get_template
from django.utils import simplejson
from django.utils.encoding import smart_str
from django.utils.translation import ugettext as _
from django.views.generic import list_detail
from django.contrib.admin.views import decorators
from django.conf import settings
import django.conf.urls.defaults as urls

# imports from aircable libs
from net.aircable.openproximity.pluginsystem import pluginsystem
from net.aircable.utils import get_subclass, get_subclasses

from net.aircable.utils import getLogger
logger = getLogger(__name__)

# imports from op libs
import models

class Node(object):
    '''
    A wrapper class for easily creating an ajax node.
    '''
    def __init__(self, id, data, state="closed", klass=None):
        self.id = id
        self.data = data
        self.state = state
        self.klass = klass
    
    def getDict(self):
        out = {
            "attr": { 
                "id": self.id 
            },
            "data": self.data
        }
        if self.state: out['state'] = self.state
        if self.klass: out["attr"]['class'] = self.klass
        return out

# urls or ids can be of two forms:
# AppLabel_ModuleName_pk
# AppLabel_ModuleName_RelatedField_RelatedPk
# AppLabel_ModuleName_RelatedField_RelatedPk, minPk, maxPk
def generic_URL(*args):
    return "_".join( str(k) for k in args)

def URL(obj):
    return generic_URL(obj._meta.app_label, obj._meta.module_name, obj.pk)

def URL_RELATED(klass, field_name, related):
    return generic_URL(
        klass._meta.app_label, klass._meta.module_name,
        field_name, related.pk)

class ModelNode(object):
    def __init__(self, obj):
        self.node = Node( 
            URL(obj), 
            "%s: %s" %(obj._meta.object_name, str(obj)),
            klass="deletable" if getattr(obj, 'deletable', None) else None
        ).getDict()

        self.node['children'] = [ 
            Node( 
                URL(obj)+"_"+o.name, 
                "%s: %s" % (o.name, getattr(obj, o.name)),
                None
            ).getDict() 
            for o in obj._meta.fields if not o.rel
        ]

        self.node['children'].extend([ 
                Node( URL(getattr(obj, o.name)), o.name ).getDict() 
                for o in obj._meta.fields if o.rel
        ])
    
        for related in obj._meta.get_all_related_objects():
            p = {"%s__pk" % related.field.name: obj.pk}
            if related.model.objects.filter(**p).count() > 0:
                for subk in get_subclasses(related.model):
                    qs = subk.objects.filter(**p)
                    if qs.count() > 1:
                        self.node['children'].append( 
                            Node(
                                URL_RELATED(subk, related.field.name, obj),
                                    "%s: %s items" % (
                                        subk._meta.object_name,
                                        qs.count())
                            ).getDict()  
                        )
                    elif qs.count() == 1:
                        self.node['children'].append( 
                            ModelNode(qs.get()).getDict() )
    def getDict(self):
        return self.node

class RelatedNode(object):
    def __init__(self, qs, app, model, related_field, related_pk):
        out = []
        out.append(
            Node(
                "_".join([app, model, related_field, 'count']),
                "Count: %s" % qs.count(),
                None
                ).getDict()
        )

        prev = None
        i=0
        j=0
        step = 200
    
        for curr in qs[::step]:
            if prev:
                url = generic_URL(app, model, related_field, related_pk, prev, 
                    curr.pk)
                title = "From %s to %s" % (i+1, j)
                out.append( Node(url, title).getDict() )

            i=j
            j+=step
            prev=curr.pk
    
        curr = qs[qs.count()-1].pk
        url = generic_URL(app, model, related_field, related_pk, prev, curr+1)
        title = "From %s to %s" % (i+1, qs.count())
        out.append( Node(url, title).getDict())
        self.children = out
    
    def getDict(self):
        return self.children


def modelList(request, klass, subclass=False):
    kl = None
    if getattr(klass, 'deletable', False):
        kl = "deletable"

    for o in klass.objects.all():
        o = get_subclass(o) if subclass else o
        yield Node( URL(o), str(o), klass=kl ).getDict()

def modelDetail(request, app, model, pk):
    o = mod.get_model(app, model).objects.get(pk=pk)
    return HttpResponse( 
        simplejson.dumps(ModelNode(o).getDict()['children']), 
        content_type="application/json")

def relatedDetail(request, app, model, rel_field, rel_pk):
    qs = mod.get_model(app, model).objects.all()
    req = {str("%s__pk" % rel_field): int(rel_pk)}
    qs = qs.filter(**req)

    return HttpResponse( 
        simplejson.dumps(RelatedNode(qs, app, model, rel_field, rel_pk)
                .getDict()), 
        content_type="application/json")

def relatedDetailSubQuery(request, app, model, rel_field, rel_pk, min_pk, max_pk):
    qs = mod.get_model(app, model).objects.all()
    req = {
        str("%s__pk" % rel_field): int(rel_pk),
        "pk__gte": int(min_pk),
        "pk__lt": int(max_pk),
    }
    qs = qs.filter(**req)

    return HttpResponse( 
        simplejson.dumps( [ ModelNode(get_subclass(o)).getDict() for o in qs] ), 
        content_type="application/json")

def wrapper(generator, *args, **kwargs):
    return HttpResponse(
        simplejson.dumps(list(generator(*args, **kwargs))),
        content_type="application/json")

def initialData(request):
    out = [
        Node("remote_devices", _("Remote Devices")).getDict(),
        Node("campaigns", _("Campaigns")).getDict(),
        Node("scanner_dongles", _("Scanner Dongles")).getDict(),
        Node("uploader_dongles", _("Uploader Dongles")).getDict(),
        Node("all_dongles", _("All Dongles")).getDict()
    ]

    return HttpResponse(
        simplejson.dumps(out),
        content_type="application/json")


DATA={
    'remote_devices': lambda x: wrapper(modelList, x, models.RemoteDevice),
    'campaigns': lambda x: wrapper(modelList, x, models.MarketingCampaign),
    'scanner_dongles': lambda x: wrapper(modelList, x, models.ScannerBluetoothDongle),
    'uploader_dongles': lambda x: wrapper(modelList, x, models.UploaderBluetoothDongle),
    'all_dongles': lambda x: wrapper(modelList, x, models.BluetoothDongle, True),
}

def data(request):
    id_ = request.POST.get('id', '').lower()
    logger.debug("treeview.data %s" % id_)
    
    if id_ in DATA:
        return DATA.get(id_)(request)
    spl = id_.split('_')
    
    if len(spl) == 3:       # we got a request for certain device
        logger.debug("model detail %s" % spl)
        return modelDetail(request, *spl)
    elif len(spl) == 4:
        logger.debug("related model detail %s" % spl)
        return relatedDetail(request, *spl)
    elif len(spl) == 6:
        logger.debug("related model detail subquery %s" % spl)
        return relatedDetailSubQuery(request, *spl)

    return initialData(request)

def delete(request):
    if request.method != 'POST':
        raise Exception("Expected a POST")

    if not request.user.is_authenticated() or not request.user.is_staff:
        return HttpResponse(
            simplejson.dumps({
                'need_login': True,
            }),
            content_type="application/json")

    app, model, pk = request.POST.get('id').split('_')
    o = mod.get_model(app, model).objects.get(pk=pk)
    o.delete()
    return HttpResponse(
        simplejson.dumps({
            'deleted': True,
            'pk': pk,
            'app': app,
            'model': model
        }),
        content_type="application/json")

def index(request):
    return render_to_response("op/treeview.html", 
        context_instance=RequestContext(request))

urlpatterns = urls.patterns('',
    (r'delete', delete),
    (r'data', data),
    (r'', index),
)



# -*- coding: utf-8 -*-
# OpenProximity2.0 is a proximity marketing OpenSource system.
# Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

# this file will be called by settings and others, before django.conf.settings
# is available

import sys, os, setpaths
__PATH=os.path.dirname(os.path.abspath(__file__))
__ROOT=os.path.dirname(__PATH)
__LIBS=os.path.join(__ROOT, 'libs')

from configglue.parser import SchemaConfigParser
from django_configglue.utils import update_settings
from functools import partial
import schema

OPENPROXIMITY_CONFIG_FILE=os.environ.get('OPENPROXIMITY_CONFIG_FILE', 
                                         "/etc/openproximity.conf")
if not os.access(OPENPROXIMITY_CONFIG_FILE, os.W_OK):
    OPENPROXIMITY_CONFIG_FILE=os.path.expanduser("~/.openproximity.conf")


# parse config files
parser=SchemaConfigParser(schema.OpenProximitySchema())
parser._interpolate = partial(schema._interpolate, parser)
parser.read([ os.path.join(__PATH, 'default.cfg'),
        os.path.join(__PATH, 'django.cfg'),
        os.path.join(__PATH, 'cherrypy.cfg'),
        os.path.join(__PATH, 'rpyc.cfg'),
        os.path.join(__PATH, 'debug.cfg'),
        OPENPROXIMITY_CONFIG_FILE])
update_settings(parser, locals())

# fix timeout in DATABASE_OPTIONS
if 'timeout' in locals()['DATABASE_OPTIONS']:
    locals()['DATABASE_OPTIONS']['timeout'] = \
        float(locals()['DATABASE_OPTIONS']['timeout'])

locals()['DEBUG_DISABLES']=locals()['DEBUG_DISABLES'].split(',')

# keep a reference to the parser
__CONFIGGLUE_PARSER__ = parser

if locals()['DEBUG_FILENAME'] == None:
    locals()['DEBUG_FILENAME'] = sys.argv[-1]

def setmode(self, mode):
    d = []
    if mode=="client":
        d = ['openproximity', 'django', 'cherrypy', 'rpyc']
    elif mode=='cherrypy':
        d = ['rpc_client', 'rpyc']
    elif mode=='rpyc':
        d = ['rpc_client', 'cherrypy']
    else:
        print "invalid mode", mode
    for i in d:
        self.schema._sections.pop(i)

parser.setmode=partial(setmode, parser)
sys.modules['preset'] = sys.modules[__name__]



#!/usr/bin/python
# -*- coding: utf-8 ; Mode: python; tab-width: 4 ; indent-tabs-mode: nil -*-
#
# OpenProximity2.0 is a proximity marketing OpenSource system.
# Copyright (C) 2010 Naranjo Manuel Francisco <manuel@aircable.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
Helper classes for handling asynchronous SDP resolving and file uploading
through OBEX.
"""

import dbus, sys, gobject, os
from lxml import etree
import logging

from net.aircable.utils import getLogger
logger = getLogger(__name__)

# channel extractor
CHANNEL_XPATH=etree.XPath(
    "/record/attribute[@id='0x0004']/sequence/sequence/uuid[@value='0x0003']/../uint8/@value")

# state machine 
IDLE = 0
CREATE_DEVICE = 1
FIND_DEVICE = 2
GET_PROPERTIES = 4
DISCOVER_SERVICES = 8
DONE_SDP = 16
PAIR_IN_PROGRESS = 32

# valid uuids
def create_paired_device_cb(self, *args, **kwargs):
        logger.debug("create_paired_device_cb %s %s" % (args, kwargs))
        self.state = IDLE
        self.reply_callback()
OBEX_UUID='00001105-0000-1000-8000-00805f9b34fb'
FTP_UUID='00001106-0000-1000-8000-00805f9b34fb'

# a helper function don't pay much attention to it
def generate_arguments(*args, **kwargs):
    ''' a function to generate the long list of needed arguments '''
    out = ['/usr/bin/obexftp', '-d', kwargs['dongle'], '-r', kwargs['retries']]
    if kwargs['service']=='opp':
        out.extend(['-U', 'none', '-H'])
    out.extend(['-S', '-T', kwargs['timeout'], '-b', kwargs['target'], 
        '-B', kwargs['channel']])
    for f in kwargs['files']:
        out.extend(['-p', f])
    return [ str(x) for x in out ]

class ServiceNotProvided(Exception):
    '''
    Exception used to tell the other end when a certain service is not 
    provided by the target.
    '''
    path = None

    def __init__(self, path, *args, **kwargs):
        super(Exception, self).__init__(*args, **kwargs)
        self.path = path

    def __str__(self):
        return "ServiceNotProvided(%s)" % self.path

def property_changed_cb(name, value, path):
    '''
    Callback that will allow us to know when we get connected to the remote end.
    This can be used to tell where in the process we loose connection. Some
    devices don't correctly make a difference between a timeout and a rejected
    so we can use a timer and this to know what happened (not really implemented
    totally).
    '''
    if name.lower() == "connected":
        if path in UploadTarget.targets:
           UploadTarget.targets[path].connected = bool(value)
        logger.info("connect %s to %s" % (value, path))

class UploadTarget(object):
    ''' 
        A class to represent an upload target.
        This class can handle profile resolving and obex uploading wihtout 
        blocking, AKA async. This means that instead of creating X threads to 
        upload to X targets you just create X instances of this class.
    
        You need to have a running gobject mainloop for it to work, check the 
        __main__ for a working example
    '''
    state = None
    dongle = None
    target = None
    target_path = None
    device = None
    uuid = None
    bus = None
    connected = False
    reply_callback = None
    error_callback = None
    loop = None
    
    #class variable
    targets = dict()
    listener = None
    
    def __init__(self, dongle, target, bus, loop=None):
        self.dongle = dongle
        self.target = target
        self.bus = bus
        self.state = IDLE
        self.loop = loop
        self.target_path = "%s/dev_%s" % (
                                self.dongle.object_path, 
                                self.target.replace(":", "_")
        )
        UploadTarget.targets[self.target_path]=self
        if not UploadTarget.listener:
            UploadTarget.listener = self.bus.add_signal_receiver(
                property_changed_cb, # callback
                signal_name='PropertyChanged', # signal name
                path_keyword = 'path' # filter path
            )

    def __ReturnError__(self, error):
        if callable(self.error_callback):
            self.error_callback(self, error=error, state=self.state, 
                                connected=self.connected)
        self.cleanup()
        self.state = IDLE

    def cleanup(self):
        logger.debug("Cleaning up")
        self.state = IDLE
        UploadTarget.targets.pop(self.target_path)

    def ResolveChannel(self, uuid, reply_callback, error_callback):
        self.reply_callback = reply_callback
        self.error_callback = error_callback
        self.uuid = uuid
        self.CreateDevice()
    
    def CreateDevice(self):
        logger.debug("createDevice")
        self.state = CREATE_DEVICE
        self.dongle.CreateDevice(
            self.target, 
            reply_handler=self.create_device_cb, 
            error_handler=self.create_device_err
        )
        
    def property_changed_cb(self, name, value):
        if name.lower() == "connected":
            self.connected = bool(value)
            logger.info("connect %s to %s" % (self.connected, self.target))
    
    def create_device_cb(self, *args, **kwargs):
        logger.debug("create_device_cb %s %s" % ( args, kwargs ) )
        self.FindDevice()
    
    def create_device_err(self, error):
        logger.debug("create_device_err %s" % error)
        if error.get_dbus_name().lower()!='org.bluez.error.alreadyexists':
            return self.__ReturnError__(error)
        self.FindDevice()
    
    def FindDevice(self):
        logger.debug("findDevice")
        self.state = FIND_DEVICE
        self.dongle.FindDevice(
            self.target, 
            reply_handler=self.find_device_cb, 
            error_handler=self.find_device_err)
    
    def find_device_cb(self, path):
        logger.debug("find_device_cb %s" % path)
        self.device = dbus.Interface(
            self.bus.get_object("org.bluez", path),
            "org.bluez.Device"
            )
        self.GetProperties()
    
    def find_device_err(self, error):
        logger.debug("find_device_error %s" % error)
        return self.__ReturnError__(error)
    
    def GetProperties(self):
        logger.debug("GetProperties")
        self.state = GET_PROPERTIES
        self.device.GetProperties(
            reply_handler = self.get_properties_cb,
            error_handler = self.get_properties_err
        )
    
    def get_properties_cb(self, properties):
        logger.debug("get_properties_cb")
        if not self.uuid in properties['UUIDs']:
            return self.__ReturnError__(ServiceNotProvided(self.target))
        self.DiscoverServices()
    
    def get_properties_err(self, error):
        logger.debug("get_properties_err %s" % error)
        return self.__ReturnError__(error)
    
    def DiscoverServices(self):
        logger.debug("DiscoverServices")
        self.state = DISCOVER_SERVICES
        self.device.DiscoverServices(
            self.uuid,
            reply_handler = self.discover_services_cb,
            error_handler = self.discover_services_err
        )
    
    def discover_services_cb(self, services):
        logger.debug("discover_services_cb")
        for key in services.keys():
            root=etree.XML(str(services[key]))
            res=CHANNEL_XPATH(root)
            if len(res)>0:
                return self.FoundService(int(res[0], 16))
        e=Exception("Channel not resolved")
        return self.__ReturnError__(e)
    
    def discover_services_err(self, error):
        logger.debug("discover_services_err %s" % error)
        return self.__ReturnError__(error)
    
    def FoundService(self, channel):
        logger.info("%s found_channel at %s" %( self.target, channel ))
        self.state = DONE_SDP
        if callable(self.reply_callback):
            self.reply_callback(self, channel)

    def PairDevice(self, address, agent, reply_callback, error_callback):
        self.reply_callback = reply_callback
        self.error_callback = error_callback
        self.CreatePairedDevice(address, agent)

    def CreatePairedDevice(self, target, agent):
        logger.debug("createPairedDevice")
        self.state = PAIR_IN_PROGRESS
        self.dongle.CreatePairedDevice(
            target,
            agent,
            "NoInputNoOutput",
            reply_handler=self.create_paired_device_cb, 
            error_handler=self.create_paired_device_err,
            timeout=6000 # so we don't get timeouts so soon!
        )
    
    def create_paired_device_cb(self, *args, **kwargs):
        logger.debug("create_paired_device_cb %s %s" % (args, kwargs))
        self.state = IDLE
        self.reply_callback(target=self)

    def create_paired_device_err(self, reason):
        logger.debug("create_paired_device_err %s %s" % (reason, type(reason)))
        self.state = IDLE
        if reason.get_dbus_name().find('AlreadyExists')>-1:
            self.reply_callback(target=self)
        else:
            self.error_callback(target=self,
                            connected=self.connected,
                            error=reason.get_dbus_message(),
                            exception=reason.get_dbus_name()
                            )

    def SendFiles(self, 
                    files, 
                    retries=1, 
                    timeout=30, 
                    channel=None,
                    service='opp', 
                    uuid=OBEX_UUID, 
                    reply_callback=None, 
                    error_callback=None):
    
        self.reply_callback = reply_callback
        self.error_callback = error_callback
    
        logger.debug("SendFiles %s %s" % (service, channel) )
        
        arguments = list(generate_arguments(
                                dongle=self.dongle.GetProperties()['Address'],
                                retries = retries, 
                                timeout = timeout, 
                                target = self.target,
                                channel =  channel, 
                                files = files, 
                                service = service
                    ))
        logger.debug("Running: %s" % ' '.join(arguments))
        b = SpawnAplication(arguments)
        b.connect("program-completed", self.send_files_cb)
    
    def send_files_cb(self, sender, pid, retcode, stdout, stderr):
        logger.debug("send_files_cb %s" % retcode)
        
        if retcode==0 or retcode==255:
            logger.info("file uploaded")
            logger.debug(stdout)
            self.reply_callback(self, stdout, stderr)
        else:
            self.error_callback(
                target=self, 
                retcode=retcode, 
                stdout=stdout, 
                stderr=stderr,
                connected=self.connected
            )
        self.cleanup()


class SpawnAplication(gobject.GObject):
    ''' 
        A simple class that starts an external application with no stdin
        lets it run, and then when it completes launches a signal
        letting the listeners what was the output of it.
        It needs the gobject event loop to avoid locking
    '''
    # based on http://paste.ubuntu.com/191521/ 
    
    def __init__(self, argv, now=True):
        super(SpawnAplication, self).__init__()
        self.argv = argv
        if now:
            self.Run()
    
    def Run(self):
        self.pid, stdin, self.stdout, self.stderr = gobject.spawn_async(
            self.argv,
            flags = gobject.SPAWN_DO_NOT_REAP_CHILD, 
            # make sure we can handle it's exit
            standard_input=False,
            standard_output=True,
            standard_error=True
            )   
        gobject.child_watch_add(self.pid, self.__HandleExit)
    
    def __HandleExit(self, pid, status):
        def read_and_close(io):
            t = gobject.IOChannel(io)
            out = t.read()
            t.close()
            return out

        def get_retcode(status):
            if os.WIFEXITED(status):
                # completed correctly calling exit
                return os.WEXITSTATUS(status)
            elif os.WIFSIGNALED(status):
                logger.debug("exited with a signal")
                return 0xff00 + os.WTERMSIG(status)
            else:
                logger.debug("unknown return condition")
                return 0xffff

        retcode = get_retcode(status)
        logger.debug("HandleExit pid: %s, retcode: %s" % (pid, retcode) )

        stdout = read_and_close(self.stdout)
        stderr = read_and_close(self.stderr)

        self.emit("program-completed", self.pid, retcode, stdout, stderr)
        return False

gobject.signal_new(
    "program-completed", 
    SpawnAplication,
    gobject.SIGNAL_RUN_LAST,
    gobject.TYPE_NONE,
    (
        gobject.TYPE_INT, 
        gobject.TYPE_INT, 
        gobject.TYPE_STRING, 
        gobject.TYPE_STRING
    )
)

def testSpawnApplication():
    '''
    A testing method for the spawn application class
    '''
    def ProgramCompleted(sender, pid, retcode, stdout, stderr):
        import os
        print "ProgramCompleted", pid, retcode, len(stdout.split('\n'))
        os.system("ps -ef | grep %s" % pid)
  
    def testSpawnApplication1():
        print "testSpawnApplication1"
        b = SpawnAplication(['/bin/sleep', '20'], False)
        b.connect("program-completed", ProgramCompleted)
        b.Run()
        print b.pid, b.stdout, b.stderr
        return False

    def testSpawnApplication2():
        print "testSpawnApplication2"
        b = SpawnAplication(['/bin/ls'], False)
        b.connect("program-completed", ProgramCompleted)
        b.Run()
        print b.pid, b.stdout, b.stderr
        return False

    gobject.timeout_add(1, testSpawnApplication1)
    gobject.timeout_add(10, testSpawnApplication2)
    loop.run()

if __name__=='__main__':
    import dbus.glib
    import dbus.mainloop.glib
    
    logger.setLevel(logging.DEBUG)
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    gobject.threads_init()
    dbus.glib.init_threads()
    loop=gobject.MainLoop()
    
    if len(sys.argv) < 3:
        print "Usage %s file targets" % sys.argv[0]
        sys.exit(0)

    bus = dbus.SystemBus()
    manager = dbus.Interface(bus.get_object("org.bluez", "/"), 
                             "org.bluez.Manager")
    adapter = dbus.Interface(bus.get_object("org.bluez", 
                                            manager.DefaultAdapter()),
                             "org.bluez.Adapter")
    files = [ sys.argv[1], ]
    targets = sys.argv[2:]
    
    def exit(target):
        pending.remove(target)
        if len(pending) == 0:
            print "all work done bye"
            loop.quit()

    def FileUploaded(target, *args, **kwargs):
        print "FileUploaded", args, kwargs
        exit(target)

    def FileFailed(target, *args, **kwargs):
        print "FileFailed", args, kwargs
        exit(target)

    def ChannelResolved(target, channel):
        print "ChannelResolved", channel
        target.SendFiles(
            channel=channel, 
            files=files, 
            reply_callback=FileUploaded, 
            error_callback=FileFailed
        )
  
    def ServiceNotProvided(target, error):
        print "ServiceNotProvided", error
        exit(target)

    pending = list()
    for target in targets:
        target = UploadTarget(adapter, target, bus, loop=loop)
        target.ResolveChannel(OBEX_UUID, ChannelResolved, ServiceNotProvided)
        pending.append(target)

    loop.run()




 #!/usr/bin/python
# -*- coding: utf-8 -*-
# OpenProximity2.0 is a proximity marketing OpenSource system.
# Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


'''
An advanced pairing manager that will connect to OpenProximity rpc server in to
ask which PIN code to use.

SPP mode not yet supported.
'''
import setpaths

import gobject
import sys, os
import dbus, dbus.service, dbus.mainloop.glib
import rpyc

from net.aircable.utils import getLogger
logger = getLogger(__name__)

DEFAULT_PIN=os.environ.get("PIN_CODE", "1234")
# default pin to use when no server is available.

logger.info("PIN code defaulting to %s" % DEFAULT_PIN)
server = None

PATH="/net/aircable/pairing"
# Path where we register our pairing agent.

def connect(address, port):
    '''
    This method wraps rpyc server connection, so when no server is available 
    the pairing manager will still work.
    '''
    logger.info("Connecting to %s:%s" % (address, port))
    try:
        s = rpyc.connect(address, int(port))
        return s
    except Exception, err:
        logger.error("can't connect to server")
        logger.exception(err)
    return None

def getPIN(address, dongle):
    '''
    This method will try to ask the server which PIN code to use, otherwise 
    fallback to the default PIN.
    '''
    global server
    if len(sys.argv) > 2:
        logger.info("server available")
        if not server:
            logger.info("server available")
            server=connect(sys.argv[1], sys.argv[2])
        try:
            out = server.root.getPIN(address, dongle)
            return str(out)
        except Exception, err:
            logger.error("couldn't get PIN from server")
            logger.exception(err)
    logger.info("faulting to default pin")
    return DEFAULT_PIN

def handle_name_owner_changed(own, old, new):
    '''
    When ever bluez goes up or down we will get called, we don't register here 
    though, we do it on handle_adapter_added as that's when we know which 
    dongles nare available.
    '''
    if not own.startswith('org.bluez'):
	    return
    
    if new is None or len(str(new))==0:
        logger.info( "bluez has gone down, time to get out")
	else:
        logger.info( "bluez started, time to restart")
    

def registerAgent(path):
    '''
    Register an agent on the given dbus path.
    '''
    adapter = dbus.Interface(bus.get_object("org.bluez", path),
                        "org.bluez.Adapter")
    adapter.RegisterAgent(PATH, "NoInputNoOutput") 
    # should we change this to NoInputNoOutput?
    logger.info("adapter registered for path %s" % path)


def handle_adapter_added(path, signal):
    '''
    When ever a new dongle is attached, or bluez starts a signal is generated, 
    that's when we register the agent for that adapter.
    '''
    logger.info("bluez.%s: %s" % (signal, path))
    registerAgent(path)

def handle_adapter_removed(path, signal):
    '''
    When ever a dongle is removed we will get notified, as we only run with one
    agent instance we have nothing to do.
    '''
    logger.info("adapter removed %s" % path)

class Rejected(dbus.DBusException):
    '''
    When the passkeys don't match BlueZ expects us to raise an exception.
    '''
    _dbus_error_name = "org.bluez.Error.Rejected"

class Agent(dbus.service.Object):
    '''
    The passkey agent it self.
    
    Documentation for the methods come from BlueZ docs.
    '''
    exit_on_release = True

    def set_exit_on_release(self, exit_on_release):
        self.exit_on_release = exit_on_release

    @dbus.service.method("org.bluez.Agent", in_signature="", out_signature="")
    def Release(self):
        '''
        Release the agent, and exit the loop. Gets called by BlueZ when it 
        needs us to release the agent.
        '''
        logger.info("Agent Release")
        if self.exit_on_release:
	        logger.info("Exiting from loop")
	        mainloop.quit()

    @dbus.service.method("org.bluez.Agent", in_signature="os", out_signature="")
    def Authorize(self, device, uuid):
        '''
        This method gets called when the service daemon needs to authorize a 
        connection/service request.
        Don't raise any exception so BlueZ knows we accept.
        '''
        logger.info("Authorize (%s, %s)" % (device, uuid))

    @dbus.service.method("org.bluez.Agent", in_signature="o", out_signature="s")
    def RequestPinCode(self, path):
        '''
        This method gets called when the service daemon needs to get the 
        passkey for an authentication.

        The return value should be a string of 1-16 characters length. The 
        string can be alphanumeric.
        '''
        device = dbus.Interface(bus.get_object("org.bluez", path), 
                            "org.bluez.Device")
        dongle = dbus.Interface(bus.get_object(
		        "org.bluez", device.GetProperties()['Adapter']),
	        "org.bluez.Adapter")
        device=str(device.GetProperties()['Address'])
        dongle=str(dongle.GetProperties()['Address'])
        pin=getPIN(device, dongle)
        logger.info("RequestPinCode (%s->%s): %s" % (dongle, device, pin) )
        return pin

    @dbus.service.method("org.bluez.Agent", in_signature="o", out_signature="u")
    def RequestPasskey(self, device):
        '''
        This method gets called when the service daemon needs to get the 
        passkey for an authentication.

        The return value should be a numeric value between 0-999999.
        '''
        logger.info("RequestPasskey (%s): %s" % (device, PIN) )
        return dbus.UInt32(PIN)

    @dbus.service.method("org.bluez.Agent", in_signature="ou", out_signature="")
    def DisplayPasskey(self, device, passkey):
        '''
        This method gets called when the service daemon needs to display a 
        passkey for an authentication.

        The entered parameter indicates the number of already typed keys on the 
        remote side.

        An empty reply should be returned. When the passkes needs no longer to 
        be displayed, the Cancel method of the agent will be called.

        During the pairing process this method might be called multiple times 
        to update the entered value.
        '''
        logger.info("DisplayPasskey (%s, %d)" % (device, passkey))

    @dbus.service.method("org.bluez.Agent", in_signature="ou", out_signature="")
    def RequestConfirmation(self, device, passkey):
        '''
        This method gets called when the service daemon needs to confirm a 
        passkey for an authentication.

        To confirm the value it should return an empty reply or an error in 
        case the passkey is invalid.
        '''
        logger.info("RequestConfirmation (%s, %d)" % (device, passkey))
        device = dbus.Interface(bus.get_object("org.bluez", device),
                        "org.bluez.Device")
        dongle = dbus.Interface(bus.get_object("org.bluez", 
                            device.GetProperties()['Adapter']),
                "org.bluez.Adapter")
        device=str(device.GetProperties()['Address'])
        dongle=str(dongle.GetProperties()['Address'])
        pin=getPIN(device, dongle)
        if passkey == pin:
	        logger.info("passkey matches")
	        return
        logger.info("passkey doesn't match")
        raise Rejected("Passkey doesn't match")

    @dbus.service.method("org.bluez.Agent", in_signature="s", out_signature="")
    def ConfirmModeChange(self, mode):
        '''
        This method gets called if a mode change is requested that needs to be 
        confirmed by the user. An example would be leaving flight mode.
        '''
        logger.info("ConfirmModeChange (%s)" % (mode))

    @dbus.service.method("org.bluez.Agent", in_signature="", out_signature="")
    def Cancel(self):
        '''
        This method gets called to indicate that the agent  request failed 
        before a reply was returned.
        '''
        logger.info("Cancel")

def registerControlSignals(bus):
    '''
    Register our selves with some important dbus signals.
    '''
    bus.add_signal_receiver(handle_name_owner_changed,
            'NameOwnerChanged',
            'org.freedesktop.DBus',
            'org.freedesktop.DBus',
            '/org/freedesktop/DBus')

    bus.add_signal_receiver(handle_adapter_added,
            signal_name='AdapterAdded',
            dbus_interface='org.bluez.Manager',
            member_keyword='signal')

    bus.add_signal_receiver(handle_adapter_removed,
            signal_name='AdapterRemoved',
            dbus_interface='org.bluez.Manager',
            member_keyword='signal')

def initAgent():
    '''
    Initialize the agent, register with every available path if possible.
    '''
    try:
		manager = dbus.Interface(bus.get_object("org.bluez", "/"),
                                    "org.bluez.Manager")
		for path in manager.ListAdapters():
			registerAgent(path)
            logger.info("Agent registered on all paths")
    except Exception, err:
		logger.error("Something went wrong on the agent application")
        logger.exception(err)

if __name__ == '__main__':
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

    bus = dbus.SystemBus()
    registerControlSignals(bus)
    mainloop = gobject.MainLoop()
    agent = Agent(bus, PATH)
    agent.set_exit_on_release(False)
    initAgent()
    mainloop.run()
    logger.info("Agent is exiting")



#!/usr/bin/env python

from django_cpserver.management.commands.runcpserver import CPSERVER_OPTIONS
from django.conf import settings
from optparse import OptionParser
from configglue.glue import schemaconfigglue


CPSERVER_OPTIONS.update({
    'host': settings.CP_HOST,
    'port': settings.CP_PORT,
    'server_name': settings.CP_SERVER_NAME,
    'threads': settings.CP_THREADS,
    'daemonize': settings.CP_DAEMONIZE,
    'workdir': settings.CP_WORKDIR,
    'pidfile': settings.CP_PIDFILE,
    'server_user': settings.CP_SERVER_USER,
    'server_group': settings.CP_SERVER_GROUP,
    'ssl_certificate': settings.CP_SSL_CERTIFICATE,
    'ssl_private_key': settings.CP_SSL_PRIVATE_KEY,
})

CPSERVER_HELP = r"""
  Run this project in a CherryPy webserver. To do this, CherryPy from
  http://www.cherrypy.org/ is required.
"""


from django_cpserver.management.commands.runcpserver import Command as OCommand

class Command(OCommand):
    def create_parser(self, prog_name, subcommand):
        """
        Add all our SchemaConfigParser's options so they can be shown
        in help messages and such.
        """
        parser = OptionParser(prog=prog_name,
            usage=self.usage(subcommand),
            version=self.get_version(),
            option_list=self.option_list)

        configglue_parser = settings.__CONFIGGLUE_PARSER__
        configglue_parser.setmode("cherrypy")
        op, options, args = schemaconfigglue(configglue_parser, op=parser)
        return op
        
    def usage(self, subcommand):
        return ""

    def handle(self, *args, **options):
        from django.conf import settings
        from net.aircable import utils
        utils.log_setup(settings, True)
        OCommand.handle(self, *args, **options)



"""
Some OpenProximity context variables that are some times
required
"""
import os
from django.conf import settings
try:
    from openproximity import __version__ as opversion
except:
    opversion = 'ND'

def variables(requesst):
    return {
        'version': { 'current': opversion },
        'settings': {
            'logo': settings.OP_LOGO,
            'debug': settings.OP_DEBUG,
            'translate': settings.OP_TRANSLATE,
            'twitter': settings.OP_TWITTER
        }
    }



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import threading, time, traceback, sys, os
from django.core.management.base import BaseCommand
from django.conf import settings
from optparse import make_option, OptionParser
from configglue.glue import schemaconfigglue

CLIENT_OPTIONS = {
    'host': settings.CLIENT_HOST,
    'port': settings.CLIENT_PORT,
    'mode': settings.CLIENT_MODE,
    'daemonize': settings.CLIENT_DAEMONIZE,
    'pidfile': settings.CLIENT_PIDFILE,
}

class Command(BaseCommand):
    help = "RPyC Client for OpenProximity."

    def handle(self, *args, **options):
        from django.conf import settings
        from net.aircable import utils
        utils.log_setup(settings, True)
        runclient(args)

    def usage(self, subcommand):
        return self.help

    def create_parser(self, prog_name, subcommand):
        """
        Add all our SchemaConfigParser's options so they can be shown
        in help messages and such.
        """
        parser = OptionParser(prog=prog_name,
            usage=self.usage(subcommand),
            version=self.get_version(),
            option_list=self.option_list)

        cg_parser = settings.__CONFIGGLUE_PARSER__
        cg_parser.setmode("client")
        op, options, args = schemaconfigglue(cg_parser, op=parser)
        return op

def runclient(argset=[], **kwargs):
    # Get the options
    options = CLIENT_OPTIONS.copy()
    options.update(kwargs)

    from net.aircable.utils import getLogger
    logger = getLogger(__name__)

    if options['daemonize']:
        if not options['pidfile']:
            options['pidfile'] = '/var/run/openproximity_client_%s_%s.pid' % ( \
                options['mode'], options['port'] )

        logger.info("storing pid file in %s" % options['pidfile'])
        from django.utils.daemonize import become_daemon
        
        logger.info("daemonizing")
        become_daemon()
        logger.info("daemonized")

        fp = open(options['pidfile'], 'w')
        fp.write("%d\n" % os.getpid())
        fp.close()

    # Start the rpc server
    logger.debug('starting client with options %s' % options)
    mode = options['mode']
    host = options['host']
    port = int(options['port'])
    valid_modes=['scanner', 'uploader', 'pairing']
    from net.aircable.openproximity.pluginsystem import pluginsystem
    valid_modes.extend([ \
            i.provides['serverxr_type'] for i in \
                pluginsystem.get_plugins('serverxr')])
    if mode not in valid_modes:
        print "not a valid mode, use any of", valid_modes
        sys.exit(1)
    from rpc_clients import manager
    manager.run(host, port, mode)

if __name__ == '__main__':
    runclient(sys.argv[1:])



# -*- coding: utf-8 -*-
# OpenProximity2.0 is a proximity marketing OpenSource system.
# Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
from datetime import datetime
import time

from pickle import dumps, loads
from django.db import models
from django.conf import settings
from django.utils.translation import ugettext as _
from django.dispatch.dispatcher import Signal
from django.contrib.auth.models import User
from timezones.fields import TimeZoneField

import rpyc, time, os.path

import net.aircable.openproximity.signals.scanner as scanner

from net.aircable.fields import PickledField
from net.aircable.utils import getLogger
logger = getLogger(__name__)

from openproximity.rpc.dispatcher import Dispatcher

import errno

TIMEOUT_RET = [
    errno.ENOTCONN, # 107
]

class UserProfile(models.Model):
    user = models.ForeignKey(User, unique=True)
    timezone = TimeZoneField()

class LogLine(models.Model):
    time = models.DateTimeField(auto_now=True, verbose_name=_("time"))
    content = models.CharField(max_length=255)

class Setting(models.Model):
    name = models.CharField(max_length=40)
    value = PickledField(max_length=200)

class BluetoothDongle(models.Model):
    address = models.CharField(
        max_length=17, 
        blank=False, 
        verbose_name=_("bluetooth address")
    )
    name = models.CharField(
        max_length=100, 
        blank=True,
        verbose_name=_("identifying name")
    )
    enabled = models.BooleanField()

    def enabled_display(self):
        if self.enabled:
            return "Enabled"
        return "Disabled"
    
    def __unicode__(self):
        return "%s - %s, %s" % (
            self.address, 
            self.name, 
            self.enabled_display() 
        )

class ScannerBluetoothDongle(BluetoothDongle):
    priority = models.IntegerField()

    def __unicode__(self):
        return "Scanner: %s, %s" % (
            BluetoothDongle.__unicode__(self), 
            self.priority
        )

class RemoteScannerBluetoothDongle(ScannerBluetoothDongle):
    local_dongle = models.ForeignKey(
        ScannerBluetoothDongle, 
        related_name="remote_dongles" )

class UploaderBluetoothDongle(BluetoothDongle):
    max_conn = models.IntegerField(
        default=7,
        verbose_name=_("connections"),
        help_text=_("maximum allowed connections")
    )
    
    def __unicode__(self):
        return "Uploader: %s, %s" % (
            BluetoothDongle.__unicode__(self), 
            self.max_conn
        )

SERVICE_TYPES = (
    (0,   u'opp'),
    (1,   u'ftp'),
)

class Campaign(models.Model):

    #name is going to change
    name = models.CharField(
        max_length=100,
        verbose_name=_("campaign name")
    )
    enabled = models.BooleanField(
        default=True,
        verbose_name=_("campaign enabled")
    )
    name_filter = models.CharField(
        null=True,
        max_length=30,
        blank=True,
        verbose_name=_("name filter")
    )
    addr_filter = models.CharField(
        null=True, 
        max_length=20, 
        blank=True,
        verbose_name=_("address filter")
    )
    devclass_filter = models.IntegerField(
        null=True, 
        blank=True,
        verbose_name=_("devclass filter")
    )
    start = models.DateTimeField(
        null=True, 
        blank=True,
        help_text=_("starting date, or null to run for ever until end"),
        verbose_name=_("start date")
    )
    end = models.DateTimeField(
        null=True, 
        blank=True,
        help_text=_("ending date, or null to run for ever since start"),
        verbose_name=_("end date")
    )
    dongle_name=models.CharField(
        null=True, 
        blank=True, 
        max_length=100,
        verbose_name=_("dongles names"),
        help_text=_("if you want your campaign to change "
        "the bluetooth dongles names when running then "
        "set this variable"),
    )
    pin_code=models.CharField(
        null=True, 
        blank=True, 
        max_length=16,
        default="1234",
        help_text=_("certain target devices require pairing"
            ", this is the pin code going to be used"),
        verbose_name=_("PIN code")
    )
    concurrent_scanning=models.BooleanField(
        default=False,
        help_text=_("concurrent scanning makes all your "
            "scanners to run simultaneous, instead of doing so in a "
            "sequence one after the other, if you enable this scanner "
            "priority is ignored"),
        verbose_name=_("concurrent scanning")
    )


    def matches(self, remote, *args, **kwargs):
        return False

    def __unicode__(self):
        return self.name

    class Meta:
        # don't create a table for me please
        abstract = True
        ordering = ['start', 'end', 'name']

class MarketingCampaign(Campaign):
    service = models.IntegerField(default=0, choices=SERVICE_TYPES)
    rejected_count = models.IntegerField(
        default=2,
        help_text=_("how many times it should try again when rejected, -1 "
            "infinite"),
        verbose_name=_("rejected count")
    )
    rejected_timeout = models.IntegerField(
        default=-1,
        help_text=_("how much time to wait after a certain "
        "device has rejected a file before we "
        "try again"),
        verbose_name=_("rejected timeout")
    )
    tries_count = models.IntegerField(
        default=-1,
        help_text=_("how many times it should try to send "
        "when timing out, -1 infinite"),
        verbose_name=_("tries count")
    )
    tries_timeout = models.IntegerField(
        default=0,
        help_text=_("how much time to wait after a certain "
        "device has made a timeout before we try again"),
        verbose_name=_("tries timeout")
    )
    accepted_count = models.IntegerField(
        default=-1,
        help_text=_("how many times will this campaign be "
        "accepted before disabling, -1 means infinite"),
        verbose_name=_("accepted count")
    )
    rssi_min = models.IntegerField(
        null=True, 
        blank=True,
        help_text=_("if the meassured rssi is over or equal"
        " than this value then campaign will match, "
        "take into account rssi is negative, range -255 "
        "0"),
        verbose_name=_("RSSI min")
    )
    rssi_max = models.IntegerField(
        null=True, 
        blank=True,
        help_text=_("if the meassured rssi is less or equal"
        " than this value then campaign will match, "
        "take into account rssi is negative, range -255"
        " 0"),
        verbose_name=_("RSSI max")
    )
    fixed_channel=models.IntegerField(
        null=True,
        blank=True,
        default = None,
        help_text=_("if you set this parameter then "
        "OpenProximity will never try to resolve sdp records and "
        "use only this channel, leave it empty unless you know "
        "what you're doing."),
        verbose_name=_("fixed channel")
    )
    upload_on_discovered = models.BooleanField(
        default=False,
        help_text=_("in some cases you may want to force "
            "that the same dongle that discovers a device is "
            "the one that's going to do the upload. Make sure "
            "all your dongles can work as scanner and uploaders "
            "otherwise you will not service some devices."),
        verbose_name=_("scan and upload on same dongle")
    )


    def __unicode__(self):
        return "MarketingCampaign: %s" % self.name

    def getTimeoutCount(self, remote):
        ''' this function will count how many times a certain devices did a
            timeout
        '''
        return RemoteBluetoothDeviceFilesRejected.objects.\
            filter(remote=remote, campaign=self).\
            filter(ret_value__in=TIMEOUT_RET).count()

    def getRejectedCount(self, remote):
        ''' this function will count how many time the user has rejected'''
        return RemoteBluetoothDeviceFilesRejected.objects.\
            filter(remote=remote, campaign=self).\
            exclude(ret_value__in=TIMEOUT_RET).count()

    def getTriesCount(self, remote):
        return RemoteBluetoothDeviceFileTry.\
            objects.filter(remote=remote, campaign=self).count()

    def hasAccepted(self, remote):
        return RemoteBluetoothDeviceFilesSuccess.objects.filter(
            campaign=self, remote=remote).count()>0

    def getAcceptedCount(self):
        return RemoteBluetoothDeviceFilesSuccess.\
            objects.filter(campaign=self).count()

    
    def __getRecordForRemote(self, remote):
        '''
        Internal method used to recover the record needed to match
        the given campaign, when trying to determinate if we should try
        again or not on this device.
        '''
        pair_tries = RemoteBluetoothDevicePairing.objects.filter(
            remote=remote).order_by('-time')[:1]
        file_tries = RemoteBluetoothDeviceFilesRejected.objects.filter(
            campaign=self, remote=remote).order_by('-time')[:1]
        if pair_tries.count() == 0 and files_tries.count() == 0:
            return None
        if pair_tries.count() == 0:
            return files_tries[0]
        if file_tries.count() == 0:
            return pair_tries[0]
        
        # we want to return the oldest record
        if pair_tries[0].time > file_tries[0].time:
            return pair_tries[0]
        return file_tries[0]


    def tryAgain(self, record=None, remote=None):
        assert record or remote, "Can't pass both record and remote as none"

        if remote:
            record = self.__getRecordForRemote(remote)
            if not record:
                logger.info("first time ever")
                return True

            logger.debug("got record, %s" % record)

        previous=time.mktime(record.time.timetuple())-time.timezone
        delta = time.mktime(time.gmtime())-time.timezone-previous
        logger.info("delta: %s" % delta)

        if record.isTimeout():
            logger.info("record timeout")
            if self.tries_count == -1:
                #ok we have to try for ever
                #is it time all ready?
                if delta >= self.tries_timeout:
                    return True
            elif self.tries_count > self.getTriesCount(record.remote):
                #ok we still need to keep trying, is it time all ready?
                if delta>=self.tries_timeout:
                    return True
            else:
                # we reached the tries count
                return False

            # reschedule to try in the future in case the device is non 
            # discoverable any more
            next_time = previous + self.tries_timeout
        else:
            logger.info("record rejected")
            if self.rejected_count == -1:
                #ok we have to reject for ever
                #is it time all ready?
                if delta >= self.rejected_timeout:
                    return True
            elif self.rejected_count > self.getRejectedCount(record.remote):
                #ok we still need to keep trying, is it time all ready?
                if delta>=self.rejected_timeout:
                    return True
            else:
                # we reached the rejected count
                return False
            next_time = previous + self.rejected_timeout

        # register into the dispatcher for later operation
        address = record.remote.address
        c = Dispatcher.cueue
        if not True in [ "remote_address" in b.kw and \
                "klass" in b.kw and \
                b.kw['remote_address'] == address and \
                b.kw['klass'] == MarketingCampaign for b in c]:
            Dispatcher.callLater(next_time, 
                schedule_try_upload, 
                klass=MarketingCampaign,
                remote_address=record.remote.address, 
                record_id=record.pk,
                rule_id=self.pk)

        return False

    def matches(self, remote, record=None, *args, **kwargs):
        '''
            This function will get called to ask the campaign
            if it matches the definition rules
        '''
        logger.info("matches %s", self.pk)

        # test is we reached the rejected count
        rejected_pass = remote is None or (
            self.rejected_count == -1 or 
            self.getRejectedCount(remote) < self.rejected_count
        )

        logger.debug("rejected_pass %s" % rejected_pass)
        if not rejected_pass:
            return False

        # test for successful uploads
        accepted_pass = self.accepted_count == -1 or (
            self.accepted_count > self.getAcceptedCount()
        )
        logger.debug("accepted_pass %s" % accepted_pass)
        if not accepted_pass:
            return False

        name_pass = self.name_filter is None or \
            remote.name is None or \
            remote.name.startswith(self.name_filter)
        logger.debug("name_pass: %s" % name_pass)
        if not name_pass:
            return False

        addr_pass = self.addr_filter is None or \
            remote.address.startswith(self.addr_filter)
        logger.debug("addr_pass: %s" % addr_pass)
        if not addr_pass:
            return False
        
        class_pass = self.devclass_filter is None or \
            (remote.devclass & self.devclass_filter)>0
            
        logger.debug("class_pass: %s" % class_pass)
        if not class_pass:
            return False

        rssi_test = not(record is None or not hasattr(record, 'getAverageRSSI'))
        logger.debug("rssi_test: %s" % rssi_test)
        
        if rssi_test:
            rssi = record.getAverageRSSI()
            rssi_pass = \
                (self.rssi_min is None or rssi >= self.rssi_min ) \
                    and \
                (self.rssi_max is None or rssi <= self.rssi_max )
            logger.info("rssi_pass %s, average: %s, min: %s, max: %s" %
                    (rssi_pass, rssi, self.rssi_min, self.rssi_max)
            )
            if not rssi_pass:
                return False

        logger.info("passed all tests")
        return True

def campaign_file_upload_to(instance, filename):
    return os.path.join('campaign', time.strftime("%Y_%m_%d__%H_%M_%S"),
        filename)

class CampaignFile(models.Model):
    chance = models.DecimalField(
        null=True, 
        blank=True, 
        default=1.0, 
        decimal_places=2, 
        max_digits=3,
        help_text=_("if < 1 then a random number generator will check if "
            "the user is lucky enough to get this file")
    )
    file = models.FileField(
        upload_to=campaign_file_upload_to,
        help_text=_("campaign file itself")
    )
    
    campaign = models.ForeignKey(MarketingCampaign)
    
    def __unicode__(self):
        return "%s: %.2f" % (self.file, self.chance)

class RemoteDevice(models.Model):
    address = models.CharField(
        max_length=17, 
        blank=False,
        verbose_name=_("bluetooth address")
    )
    name = models.CharField(
        max_length=100, 
        blank=True, 
        null=True,
        verbose_name=_("identifying name")
    )
    last_seen = models.DateTimeField(
        auto_now=True, 
        blank=False,
        verbose_name=_("time")
    )
    devclass = models.IntegerField(null=True)
    
    def __unicode__(self):
        return "%s, %s" % (self.address, self.name)

    @classmethod
    def getRemoteDevice(cls, address, name=None, devclass=None):
        qs = cls.objects.filter(address=address)
        if qs.count() == 0:
            logger.info("first time found, not yet known in our DB") 
            remote = RemoteDevice()
            remote.address = address
            remote.name = name
            remote.devclass = devclass
            remote.save()
            return remote
        if qs.count() > 1:
            for i in qs[:1]:
                i.delete()
        return qs[0]
RemoteDevice.deletable = True # needed for treeview


class DeviceRecord(models.Model):
    time = models.DateTimeField(
        blank=False, 
        serialize = True,
        verbose_name=_("time")
    )
    dongle = models.ForeignKey(BluetoothDongle, 
        blank=True, 
        null=True, 
        serialize = True,
        verbose_name=_("dongle address")
    )

    def __unicode__(self):
        return self.dongle.address
    
    def nodeRepresentation(self):
        return "%s - %s [%s]" % (
            str(self.time), 
            self.dongle.address, 
            self._meta.module_name
        )

    class Meta:
        ordering = ['time']

    def save(self, force_insert=False, force_update=False):
        if self.time is None:
            self.time = datetime.utcnow()
        super(DeviceRecord, self).save(force_insert, force_update)
DeviceRecord.deletable = True # needed for treeview

class RemoteBluetoothDeviceRecord(DeviceRecord):
    remote = models.ForeignKey(
        RemoteDevice, 
        verbose_name=_("remote address"), 
        serialize = True
    )

    def setRemoteDevice(self, address):
        self.remote = RemoteDevice.getRemoteDevice(address)

    def __unicode__(self):
        return "%s, %s" % (
            self.dongle.address, 
            self.remote.address
        )

    def nodeRepresentation(self):
        return "%s - %s->%s [%s]" % (
            str(self.time), 
            self.dongle.address, 
            self.remote.address, 
            self._meta.module_name
    )

    class Meta:
        ordering = ['time']

ACTIONS = [
    'sdp-resolving',
    'pairing',
    'upload'
]

ACTIONS = map( lambda x,y: (x,y), range(len(ACTIONS)), ACTIONS)

class RemoteBluetoothDevicePendingAction(RemoteBluetoothDeviceRecord):
    '''
    A class that describes current and past actions made by OpenProximity
    '''
    action = models.IntegerField(
        choices = ACTIONS,
        help_text="Kind of action taken"
    )
    end = models.DateTimeField( null = True,
        help_text="time when this action completed"
    )
    length = models.FloatField(
        null=True,
        help_text="time it took this action to complete in seconds"
    )

    def endAction(self):
        self.end = datetime.utcnow()
        self.length = (self.end-self.time).total_seconds()


class RemoteBluetoothDeviceFoundRecord(RemoteBluetoothDeviceRecord):
    __rssi = models.CommaSeparatedIntegerField(
        max_length=200, 
        verbose_name=_("rssi"), 
        serialize = True
    )

    def setRSSI(self, rssi):
        self.__rssi = str(rssi).replace('[','').replace(']','')
    
    def getRSSI(self):
        return [ int(a) for a in self.__rssi.split(",") ]

    def getAverageRSSI(self):
        rssi=self.getRSSI()
        return sum(rssi)/float(len(rssi))

    def __unicode__(self):
        return "%s, %s, %s" % (
            self.dongle.address, 
            self.remote.address,
            self.__rssi
        )

class RemoteBluetoothDeviceSDP(RemoteBluetoothDeviceRecord):
    channel = models.IntegerField(
        help_text=_("bluetooth rfcomm channel that provides the used service"))

    def __unicode__(self):
        return "%s, %s, %s" % (
            self.remote.address,
            self.remote.name, 
            self.channel
        )

class RemoteBluetoothDeviceNoSDP(RemoteBluetoothDeviceRecord):
    pass

class RemoteBluetoothDeviceSDPTimeout(RemoteBluetoothDeviceRecord):
    pass

PAIRING = ['Passed', 'Rejected', 'Timeout']
PAIRING = map(lambda x,y: (x,y), range(len(PAIRING)), PAIRING)

class RemoteBluetoothDevicePairing(RemoteBluetoothDeviceRecord):
    '''
    A class to describe a pairing action
    '''
    state = models.IntegerField(
        choices = PAIRING
    )
    msg = models.CharField( null = True, blank = True, max_length = 50)
    exception = models.CharField( null = True, blank = True, max_length = 50 )

    # --- methods that emulate RemoteBluetoothDeviceFileTry API ---
    def isTimeout(self):
        return self.state == self.TIMEOUT

    # --- end of api mimic methods ---

    @classmethod
    def __count(klass, remote, state=None):
        qs = klass.objects.all()
        if state:
            qs=qs.filter(state=state)
        if isinstance(remote, str) or isinstance(remote, unicode):
            return qs.filter(remote__address=remote)
        else:
            return qs.filter(remote=remote)

    @classmethod
    def isPaired(klass, remote):
        logger.debug("isPaired %s" % remote )
        return klass.__count(remote, klass.PAIRED).count() > 0
    
    @classmethod
    def rejectedCount(klass, remote):
        logger.debug("rejectedCount %s" % remote)
        return klass.__count(remote=remote, state=klass.REJECTED).count()

    @classmethod
    def timeoutCount(klass, remote):
        logger.debug("timeoutCount %s" % remote)
        return klass.__count(remote=remote, state=klass.TIMEOUT).count()

    @classmethod
    def triesCount(klass, remote):
        logger.debug("triesCount %s" % remote)
        return klass.__count(remote=remote).count()

    @classmethod
    def getDonglesForRemote(klass, remote):
        logger.debug("getDonglesForRemote %s" % remote)

        return [ a['dongle__address'] for a in 
            klass.__count(remote=remote).values('dongle__address') ]

STATES = dict([(a[1], a[0]) for a in PAIRING])
RemoteBluetoothDevicePairing.STATES = STATES
RemoteBluetoothDevicePairing.PAIRED = STATES['Passed']
RemoteBluetoothDevicePairing.REJECTED = STATES['Rejected']
RemoteBluetoothDevicePairing.TIMEOUT = STATES['Timeout']

class RemoteBluetoothDeviceFileTry(RemoteBluetoothDeviceRecord):
    campaign = models.ForeignKey(MarketingCampaign)
    
    class Meta:
        ordering = ['time']
    
class RemoteBluetoothDeviceFilesRejected(RemoteBluetoothDeviceFileTry):
    ret_value = models.IntegerField()
    
    def isTimeout(self):
        return self.ret_value is not None and self.ret_value in TIMEOUT_RET
    
    def __unicode__(self):
        return "%s %s, timeout:%s" % (
            RemoteBluetoothDeviceFileTry.__unicode__(self), 
            self.ret_value, self.isTimeout()
        )

class RemoteBluetoothDeviceFilesSuccess(RemoteBluetoothDeviceFileTry):

    def save(self, *args, **kwargs):
        super(RemoteBluetoothDeviceFilesSuccess, self).save(*args, **kwargs)

        if self.campaign.accepted_count > -1:
            c = self.campaign.getAcceptedCount()
            logger.info("accepted filter: %s, count: %s" % 
                ( self.campaign.accepted_count, c ) )
            if c >= self.campaign.accepted_count:
                self.campaign.enabled = False
                self.campaign.no_restart = True
                self.campaign.save()

def getMatchingCampaigns(
        remote=None, 
        time_=None,
        enabled=None, 
        classes=None, 
        record=None):
    out  = list()
    classes = classes or Campaign.__subclasses__()
    time_ = time_ or datetime(*time.gmtime()[:-2])
    logger.info("getMatchingCampaigns %s %s %s %s %s" % ( remote, 
        time_, 
        enabled, 
        classes,
        record ) )

    for model in classes:
        rules = model.objects

        if enabled is not None:
            rules = rules.filter(enabled=enabled)

        rules = rules.all()
        rules = rules.filter(start__isnull=True) | rules.filter(start__lte=time_)
        rules = rules.filter(end__isnull=True) | rules.filter(end__gte=time_)

        if remote is None:
            out.extend(rules)
            continue

        for rule in rules:
            if rule.matches(remote, record):
                out.append(rule)

    if len(out):
        logger.info("There's a match")
        logger.debug(out)
    return out

def get_campaign_rule(files):
    logger.info('get_campaign_rule %s' % files)
    out = set()

    for file, camp_id in files:
        logger.debug(file)
        try:
            camp = MarketingCampaign.objects.get(pk=camp_id)
            logger.debug(camp)
            if len(out) > 0 and camp not in out:
                logger.error("multiple return values")
            out.add(camp)
        except Exception, err:
            logger.exception(err)
    if len(out) == 0:
        logger.info("couldn't find campaing")
        return None

    logger.info("found camp")
    return list(out)

def __restart_server():
    logger.info("restarting server")
    try:
        server = rpyc.connect('localhost', 8010)
        server.root.restart()
    except:
        #could be that we're only running the web server
        pass

def bluetooth_dongle_signal(instance, **kwargs):
    ''' gets called when ever there is a change in dongles '''
    if isinstance(instance, BluetoothDongle):
        logger.info('bluetooth_dongle_signal')
        __restart_server()

def campaign_signal(instance, **kwargs):
    ''' gets called when ever there is a change in marketing campaigns '''
    if (isinstance(instance, CampaignFile) or isinstance(instance, Campaign)) \
            and not hasattr(instance, 'no_restart'):
        logger.info('campaing_signal')
        __restart_server()

def logline_signal(instance, **kwargs):
    ''' gets called when ever there is a new logline'''
    if isinstance(instance, LogLine):
        if LogLine.objects.count()<100:
            return
        logger.info('logline cache clean up')
        logger.debug("logline count: %s" % LogLine.objects.count())
        qs = LogLine.objects.all().order_by('time')
        idm = qs[0].pk
        idM = qs[qs.count()-50].pk
        LogLine.objects.filter(pk__gte=idm, pk__lte=idM).delete()
        logger.debug("logline count: %s" % LogLine.objects.count())

models.signals.post_save.connect(bluetooth_dongle_signal)
models.signals.post_save.connect(campaign_signal)
models.signals.post_save.connect(logline_signal)

from openproximity.rpc.common import schedule_try_upload

from south.modelsinspector import add_introspection_rules
add_introspection_rules([], ["^timezones\.fields\.TimeZoneField"])
